@@ -13,14 +13,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "HWApplication.h"
 #include <QFileOpenEvent>
 #include <QEvent>
 
-#include "hwform.h"
 #include "MessageDialog.h"
 
 #if !defined(Q_OS_WIN)
