@@ -86,10 +86,11 @@ void GameUIConfig::reloadValues(void)
     else Form->ui.pageOptions->CBResolution->setCurrentIndex(t);
 
     // Default the windowed resolution to 5/6 of the screen size
-    int screenWidth = SDL_GetVideoInfo()->current_w * 5 / 6;
-    int screenHeight = SDL_GetVideoInfo()->current_h * 5 / 6;
-    QString widthStr; widthStr.setNum(screenWidth);
-    QString heightStr; heightStr.setNum(screenHeight);
+    QSize screenSize = SDLInteraction::instance().getCurrentResolution();
+    screenSize *= 5.0 / 6;
+
+    QString widthStr = QString::number(screenSize.width());
+    QString heightStr = QString::number(screenSize.height());
     QString wWidth = value("video/windowedWidth", widthStr).toString();
     QString wHeight = value("video/windowedHeight", heightStr).toString();
     // If left blank reset the resolution to the default
@@ -133,7 +134,7 @@ void GameUIConfig::reloadValues(void)
     netHost = new QString(value("net/ip", "").toString());
     netPort = value("net/port", NETGAME_DEFAULT_PORT).toUInt();
 
-    Form->ui.pageNetServer->leServerDescr->setText(value("net/servername", "hedgewars server").toString());
+    Form->ui.pageNetServer->leServerDescr->setText(value("net/servername", "Hedgewars Server").toString());
     Form->ui.pageNetServer->sbPort->setValue(value("net/serverport", NETGAME_DEFAULT_PORT).toUInt());
 
     Form->ui.pageOptions->CBShowFPS->setChecked(value("fps/show", false).toBool());
@@ -182,10 +183,9 @@ void GameUIConfig::reloadVideosValues(void)
     Form->ui.pageOptions->setDefaultOptions();
 
     // then load user configuration
-    Form->ui.pageOptions->framerateBox->setCurrentIndex(
-            Form->ui.pageOptions->framerateBox->findData(
-                        value("videorec/framerate", rec_Framerate()).toString() + " fps",
-                    Qt::MatchExactly) );
+    int framerateBoxIndex = Form->ui.pageOptions->framerateBox->findData(value("videorec/framerate", rec_Framerate()).toUInt());
+    if(framerateBoxIndex != -1)
+        Form->ui.pageOptions->framerateBox->setCurrentIndex(framerateBoxIndex);
     Form->ui.pageOptions->bitrateBox->setValue(value("videorec/bitrate", rec_Bitrate()).toUInt());
     bool useGameRes = value("videorec/usegameres",Form->ui.pageOptions->checkUseGameRes->isChecked()).toBool();
     if (useGameRes)
@@ -624,10 +624,7 @@ QRect GameUIConfig::rec_Resolution()
 
 int GameUIConfig::rec_Framerate()
 {
-    // remove the "fps" label
-    QString fpsText = Form->ui.pageOptions->framerateBox->currentText();
-    QStringList fpsList = fpsText.split(" ");
-    return fpsList.first().toInt();
+    return Form->ui.pageOptions->framerateBox->itemData(Form->ui.pageOptions->framerateBox->currentIndex()).toInt();
 }
 
 int GameUIConfig::rec_Bitrate()
