@@ -42,6 +42,7 @@ extern int cMinServerVersion;
 class QStandardItemModel;
 
 extern QString * cDefaultAmmoStore;
+extern QString * cEmptyAmmoStore;
 extern int cAmmoNumber;
 extern QList< QPair<QString, QString> > cDefaultAmmos;
 
@@ -66,13 +67,14 @@ extern int years_since_foundation;
 #define SEASON_CHRISTMAS 2
 #define SEASON_HWBDAY 4
 #define SEASON_EASTER 8
+#define SEASON_APRIL1 16
 
 #define NETGAME_DEFAULT_SERVER "netserver.hedgewars.org"
 #define NETGAME_DEFAULT_PORT 46631
 #define HEDGEHOGS_PER_TEAM 8
 
 
-// see http://en.wikipedia.org/wiki/List_of_colors
+// see https://en.wikipedia.org/wiki/List_of_colors
 /*define HW_TEAMCOLOR_ARRAY  {0xff007fff, /. azure          ./ \
                               0xffdd0000, /. classic red    ./ \
                               0xff3e9321, /. classic green  ./ \
