@@ -94,21 +94,21 @@ char sdlkeys[1024][2][128] =
     {"y", "Y"},
     {"z", "Z"},
     {"delete", QT_TRANSLATE_NOOP("binds (keys)", "Delete")},
-    {"[0]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 0")},
-    {"[1]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 1")},
-    {"[2]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 2")},
-    {"[3]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 3")},
-    {"[4]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 4")},
-    {"[5]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 5")},
-    {"[6]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 6")},
-    {"[7]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 7")},
-    {"[8]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 8")},
-    {"[9]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 9")},
-    {"[.]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad .")},
-    {"[/]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad /")},
-    {"[*]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad *")},
-    {"[-]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad -")},
-    {"[+]", QT_TRANSLATE_NOOP("binds (keys)", "Numpad +")},
+    {"keypad_0", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 0")},
+    {"keypad_1", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 1")},
+    {"keypad_2", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 2")},
+    {"keypad_3", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 3")},
+    {"keypad_4", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 4")},
+    {"keypad_5", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 5")},
+    {"keypad_6", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 6")},
+    {"keypad_7", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 7")},
+    {"keypad_8", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 8")},
+    {"keypad_9", QT_TRANSLATE_NOOP("binds (keys)", "Numpad 9")},
+    {"keypad_.", QT_TRANSLATE_NOOP("binds (keys)", "Numpad .")},
+    {"keypad_/", QT_TRANSLATE_NOOP("binds (keys)", "Numpad /")},
+    {"keypad_*", QT_TRANSLATE_NOOP("binds (keys)", "Numpad *")},
+    {"keypad_-", QT_TRANSLATE_NOOP("binds (keys)", "Numpad -")},
+    {"keypad_+", QT_TRANSLATE_NOOP("binds (keys)", "Numpad +")},
     {"enter", QT_TRANSLATE_NOOP("binds (keys)", "Enter")},
     {"equals", QT_TRANSLATE_NOOP("binds (keys)", "Equals")},
     {"up", QT_TRANSLATE_NOOP("binds (keys)", "Up")},
@@ -118,8 +118,8 @@ char sdlkeys[1024][2][128] =
     {"insert", QT_TRANSLATE_NOOP("binds (keys)", "Insert")},
     {"home", QT_TRANSLATE_NOOP("binds (keys)", "Home")},
     {"end", QT_TRANSLATE_NOOP("binds (keys)", "End")},
-    {"page up", QT_TRANSLATE_NOOP("binds (keys)", "Page up")},
-    {"page down", QT_TRANSLATE_NOOP("binds (keys)", "Page down")},
+    {"page_up", QT_TRANSLATE_NOOP("binds (keys)", "Page up")},
+    {"page_down", QT_TRANSLATE_NOOP("binds (keys)", "Page down")},
     {"f1", "F1"},
     {"f2", "F2"},
     {"f3", "F3"},
