@@ -26,6 +26,7 @@
 #include <QMessageBox>
 #include <QNetworkReply>
 #include <QDebug>
+#include <QMimeData>
 #include "hwconsts.h"
 #include "SDLInteraction.h"
 #include "SDL.h"
@@ -71,15 +72,18 @@ About::About(QWidget * parent) :
     QLabel *lbl1 = new QLabel(this);
     lbl1->setOpenExternalLinks(true);
     lbl1->setText(
-        "<style type=\"text/css\">"
-        "a { color: #ffcc00; }"
-//            "a:hover { color: yellow; }"
-        "</style>"
-        "<div align=\"center\"><h1>Hedgewars " + *cVersionString + "</h1>"
-        "<h3>" + QLabel::tr("Revision") + " " + *cRevisionString + " (" + *cHashString + ")</h3>"
-        "<p><a href=\"http://www.hedgewars.org/\">http://www.hedgewars.org/</a></p>" +
-        QLabel::tr("This program is distributed under the %1").arg("<a \
-        href=\"http://www.gnu.org/licenses/gpl-2.0.html\">GNU GPL v2</a>") +
+        //: %1 contains Hedgewars' version number
+        "<div align=\"center\"><h1>"+QString(tr("Hedgewars %1")).arg(*cVersionString) + "</h1>"
+        //: “Revision” stands for a revision in Mercurial, a distributed version control system. %1 is the revision, %2 is the hexadecimal hash.
+        "<h3>" + QString(tr("Revision %1 (%2)")).arg(*cRevisionString, *cHashString) + "</h3>"
+        //: %1 is replaced by the URL of Hedgewars.
+        "<p>" + QString(tr("Visit our homepage: %1"))
+        .arg("<a href=\"https://www.hedgewars.org/\">https://www.hedgewars.org/</a>") + "</p>" +
+        //: %1 is the name of a license
+        tr("This program is distributed under the %1.")
+	.arg("<a href=\"https://www.gnu.org/licenses/gpl-2.0.html\">"+
+        //: Short for “GNU General Public License version 2”
+        tr("GNU GPL v2")+"</a>") +
         "</div>"
     );
     lbl1->setWordWrap(true);
@@ -93,92 +97,107 @@ About::About(QWidget * parent) :
 
     /* Library information */
 
-    QString libinfo = "<style type=text/css>a:link { color: #FFFF6E; }</style>";
+    //: For the version numbers of Hedgewars' software dependencies
+    QString libinfo = QString(tr("Dependency versions:") + QString("<br>"));
 
 #ifdef __GNUC__
-    libinfo.append(QString("<a href=\"http://gcc.gnu.org\">GCC</a> %1<br>").arg(__VERSION__));
+    libinfo.append(QString(tr("<a href=\"https://gcc.gnu.org\">GCC</a>: %1")).arg(__VERSION__));
+    libinfo.append(QString("<br>"));
 #else
     libinfo.append(QString(tr("Unknown Compiler")).arg(__VERSION__) + QString("<br>"));
 #endif
 
-    const SDL_version *sdl_ver = SDL_Linked_Version();
-    libinfo.append(QString("<a href=\"http://www.libsdl.org/\">SDL</a> version: %1.%2.%3<br>")
+    const SDL_version *sdl_ver;
+    SDL_version sdl_version;
+    SDL_GetVersion(&sdl_version);
+    sdl_ver = &sdl_version;
+    libinfo.append(QString(tr("<a href=\"https://www.libsdl.org/\">SDL2</a>: %1.%2.%3"))
         .arg(sdl_ver->major)
         .arg(sdl_ver->minor)
         .arg(sdl_ver->patch));
+    libinfo.append(QString("<br>"));
 
     const SDL_version *sdlmixer_ver = Mix_Linked_Version();
-    libinfo.append(QString("<a href=\"http://www.libsdl.org/\">SDL_mixer</a> version: %1.%2.%3<br>")
+    libinfo.append(QString(tr("<a href=\"https://www.libsdl.org/\">SDL2_mixer</a>: %1.%2.%3"))
         .arg(sdlmixer_ver->major)
         .arg(sdlmixer_ver->minor)
         .arg(sdlmixer_ver->patch));
+    libinfo.append(QString("<br>"));
 
     // the remaining sdl modules used only in engine, so instead of needlessly linking them here
     // we dynamically call the function returning the linked version
-    void *sdlnet_handle = SDL_LoadObject(sopath("SDL_net"));
+    void *sdlnet_handle = SDL_LoadObject(sopath("SDL2_net"));
     if (sdlnet_handle != NULL) {
         SDL_version *(*sdlnet_ver_get)(void) = NULL;
         sdlnet_ver_get = (SDL_version *(*)(void)) SDL_LoadFunction(sdlnet_handle, "SDLNet_Linked_Version");
         if (sdlnet_ver_get != NULL) {
             SDL_version *sdlnet_ver = sdlnet_ver_get();
-            libinfo.append(QString("<a href=\"http://www.libsdl.org/\">SDL_net</a> version: %1.%2.%3<br>")
+            libinfo.append(QString(tr("<a href=\"https://www.libsdl.org/\">SDL2_net</a>: %1.%2.%3"))
                 .arg(sdlnet_ver->major)
                 .arg(sdlnet_ver->minor)
                 .arg(sdlnet_ver->patch));
+            libinfo.append(QString("<br>"));
         }
         SDL_UnloadObject(sdlnet_handle);
     }
 
-    void *sdlimage_handle = SDL_LoadObject(sopath("SDL_image"));
+    void *sdlimage_handle = SDL_LoadObject(sopath("SDL2_image"));
     if (sdlimage_handle != NULL) {
         SDL_version *(*sdlimage_ver_get)(void) = NULL;
         sdlimage_ver_get = (SDL_version *(*)(void)) SDL_LoadFunction(sdlimage_handle, "IMG_Linked_Version");
         if (sdlimage_ver_get != NULL) {
             SDL_version *sdlimage_ver = sdlimage_ver_get();
-            libinfo.append(QString("<a href=\"http://www.libsdl.org/\">SDL_image</a> version: %1.%2.%3<br>")
+            libinfo.append(QString(tr("<a href=\"https://www.libsdl.org/\">SDL2_image</a>: %1.%2.%3"))
                 .arg(sdlimage_ver->major)
                 .arg(sdlimage_ver->minor)
                 .arg(sdlimage_ver->patch));
+            libinfo.append(QString("<br>"));
         }
-        SDL_UnloadObject(sdlnet_handle);
+        SDL_UnloadObject(sdlimage_handle);
     }
 
-    void *sdlttf_handle = SDL_LoadObject(sopath("SDL_ttf"));
+    void *sdlttf_handle = SDL_LoadObject(sopath("SDL2_ttf"));
     if (sdlttf_handle != NULL) {
         SDL_version *(*sdlttf_ver_get)(void) = NULL;
         sdlttf_ver_get = (SDL_version *(*)(void)) SDL_LoadFunction(sdlttf_handle, "TTF_Linked_Version");
         if (sdlttf_ver_get != NULL) {
             SDL_version *sdlttf_ver = sdlttf_ver_get();
-            libinfo.append(QString("<a href=\"http://www.libsdl.org/\">SDL_ttf</a> version: %1.%2.%3<br>")
+            libinfo.append(QString(tr("<a href=\"https://www.libsdl.org/\">SDL2_ttf</a>: %1.%2.%3"))
                 .arg(sdlttf_ver->major)
                 .arg(sdlttf_ver->minor)
                 .arg(sdlttf_ver->patch));
+            libinfo.append(QString("<br>"));
         }
-        SDL_UnloadObject(sdlnet_handle);
+        SDL_UnloadObject(sdlttf_handle);
     }
 
 
-    libinfo.append(QString("<a href=\"http://qt-project.org/\">Qt</a> version: %1<br>").arg(QT_VERSION_STR));
+    libinfo.append(QString(tr("<a href=\"https://www.qt.io/developers/\">Qt</a>: %1")).arg(QT_VERSION_STR));
+    libinfo.append(QString("<br>"));
 
 #ifdef VIDEOREC
-    libinfo.append(QString("<a href=\"http://libav.org\">libavcodec</a> version: %1.%2.%3<br>")
+    libinfo.append(QString(tr("<a href=\"https://libav.org\">libavcodec</a>: %1.%2.%3"))
         .arg(LIBAVCODEC_VERSION_MAJOR)
         .arg(LIBAVCODEC_VERSION_MINOR)
         .arg(LIBAVCODEC_VERSION_MICRO));
-    libinfo.append(QString("<a href=\"http://libav.org\">libavformat</a> version: %1.%2.%3<br>")
+    libinfo.append(QString("<br>"));
+    libinfo.append(QString(tr("<a href=\"https://libav.org\">libavformat</a>: %1.%2.%3"))
         .arg(LIBAVFORMAT_VERSION_MAJOR)
         .arg(LIBAVFORMAT_VERSION_MINOR)
         .arg(LIBAVFORMAT_VERSION_MICRO));
-    libinfo.append(QString("<a href=\"http://libav.org\">libavutil</a> version: %1.%2.%3<br>")
+    libinfo.append(QString("<br>"));
+    libinfo.append(QString(tr("<a href=\"https://libav.org\">libavutil</a>: %1.%2.%3"))
         .arg(LIBAVUTIL_VERSION_MAJOR)
         .arg(LIBAVUTIL_VERSION_MINOR)
         .arg(LIBAVUTIL_VERSION_MICRO));
+    libinfo.append(QString("<br>"));
 #endif
 
-    libinfo.append(QString("<a href=\"http://icculus.org/physfs/\">PhysFS</a> version: %1.%2.%3<br>")
+    libinfo.append(QString(tr("<a href=\"https://icculus.org/physfs/\">PhysFS</a>: %1.%2.%3"))
         .arg(PHYSFS_VER_MAJOR)
         .arg(PHYSFS_VER_MINOR)
         .arg(PHYSFS_VER_PATCH));
+    libinfo.append(QString("<br>"));
 
     // TODO: how to add Lua information?
 
