@@ -154,7 +154,7 @@ KeyBinder::KeyBinder(QWidget * parent, const QString & helpText, const QString &
             curTable = new QTableWidget(0, 2);
             curTable->verticalHeader()->setVisible(false);
             curTable->horizontalHeader()->setVisible(false);
-            curTable->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
+            curTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
             curTable->verticalHeader()->setDefaultSectionSize(rowHeight);
             curTable->setShowGrid(false);
             curTable->setStyleSheet("QTableWidget { border: none; } ");
@@ -180,6 +180,12 @@ KeyBinder::KeyBinder(QWidget * parent, const QString & helpText, const QString &
         curTable->insertRow(row);
         curTable->setItem(row, 0, nameCell);
         QTableWidgetItem * bindCell = new QTableWidgetItem(comboBox->currentText());
+        QIcon dropDownIcon = QIcon();
+        QPixmap dd1 = QPixmap(":/res/dropdown.png");
+        QPixmap dd2 = QPixmap(":/res/dropdown_selected.png");
+        dropDownIcon.addPixmap(dd1, QIcon::Normal);
+        dropDownIcon.addPixmap(dd2, QIcon::Selected);
+        bindCell->setIcon(dropDownIcon);
         bindCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
         curTable->setItem(row, 1, bindCell);
         curTable->resizeColumnsToContents();
