@@ -47,6 +47,7 @@ class TeamSelWidget : public QGroupBox
         bool isPlaying(const HWTeam &team) const;
         QList<HWTeam> getPlayingTeams() const;
         QList<HWTeam> getNotPlayingTeams() const;
+	unsigned short getNumHedgehogs() const;
         void setInteractivity(bool interactive);
 
     public slots:
@@ -69,7 +70,7 @@ class TeamSelWidget : public QGroupBox
         void proxyTeamColorChanged(const HWTeam& team);
 
     private:
-        void addScrArea(FrameTeams* pfteams, QColor color, int maxHeight);
+        void addScrArea(FrameTeams* pfteams, QColor color, int minHeight, int maxHeight, bool setFrame);
         FrameTeams* frameDontPlaying;
         FrameTeams* framePlaying;
 
