@@ -37,6 +37,7 @@ import qualified Data.ByteString as BW
 import qualified Codec.Binary.Base64 as Base64
 import System.Process
 import Data.Maybe
+import Data.Either
 import qualified Data.List as L
 #if !defined(mingw32_HOST_OS)
 import System.Posix
@@ -54,7 +55,7 @@ data Message = Packet [B.ByteString]
     deriving Show
 
 serverAddress = "netserver.hedgewars.org"
-protocolNumber = "49"
+protocolNumber = "53"
 
 getLines :: Handle -> IO [B.ByteString]
 getLines h = g
@@ -83,6 +84,7 @@ engineListener coreChan h fileName = do
         start = flip L.elem ["WINNERS", "DRAW"]
         ps ("DRAW" : bs) = "DRAW" : ps bs
         ps ("WINNERS" : n : bs) = let c = readInt_ n in "WINNERS" : n : take c bs ++ (ps $ drop c bs)
+        ps ("GHOST_POINTS" : n : bs) = let c = 2 * (readInt_ n) in "GHOST_POINTS" : n : take c bs ++ (ps $ drop c bs)
         ps ("ACHIEVEMENT" : typ : teamname : location : value : bs) =
             "ACHIEVEMENT" : typ : teamname : location : value : ps bs
         ps _ = []
@@ -91,7 +93,7 @@ checkReplay :: String -> String -> String -> Chan Message -> [B.ByteString] -> I
 checkReplay home exe prefix coreChan msgs = do
     tempDir <- getTemporaryDirectory
     (fileName, h) <- openBinaryTempFile tempDir "checker-demo"
-    B.hPut h . BW.pack . concat . map (fromMaybe [] . Base64.decode . B.unpack) $ msgs
+    B.hPut h . B.concat . map (either (const B.empty) id . Base64.decode) $ msgs
     hFlush h
     hClose h
 
@@ -153,10 +155,12 @@ session l p home exe prefix s = do
             CheckFailed msg -> do
                 warningM "Check" "Check failed"
                 answer ["CHECKED", "FAIL", msg]
+                threadDelay 1500000
                 answer ["READY"]
             CheckSuccess msgs -> do
                 warningM "Check" "Check succeeded"
                 answer ("CHECKED" : "OK" : msgs)
+                threadDelay 1500000
                 answer ["READY"]
     where
     answer :: [B.ByteString] -> IO ()
@@ -176,7 +180,7 @@ session l p home exe prefix s = do
 
 
 main :: IO ()
-main = withSocketsDo $ do
+main = withSocketsDo . forever $ do
 #if !defined(mingw32_HOST_OS)
     installHandler sigPIPE Ignore Nothing
     installHandler sigCHLD Ignore Nothing
