@@ -29,7 +29,6 @@ import System.IO
 import qualified Data.List as List
 import Control.Monad
 import qualified Data.ByteString.Lazy as BL
-import qualified Text.Show.ByteString as BS
 import qualified Data.ByteString.Char8 as B
 import qualified Data.ByteString.UTF8 as UTF8
 import Data.Maybe
@@ -73,6 +72,7 @@ modifyTeam team room = room{teams = replaceTeam team $ teams room}
         else
             t : replaceTeam tm ts
 
+-- NOTE: Don't forget to update the error messages when you change the naming rules!
 illegalName :: B.ByteString -> Bool
 illegalName b = B.null b || length s > 40 || all isSpace s || isSpace (head s) || isSpace (last s) || any isIllegalChar s
     where
@@ -115,6 +115,12 @@ protoNumber2ver v = Map.findWithDefault "Unknown" v vermap
             , (48, "0.9.21-dev")
             , (49, "0.9.21")
             , (50, "0.9.22-dev")
+            , (51, "0.9.22")
+            , (52, "0.9.23-dev")
+            , (53, "0.9.23")
+            , (54, "0.9.24-dev")
+            , (55, "0.9.24")
+            , (56, "0.9.25-dev")
             ]
 
 askFromConsole :: B.ByteString -> IO B.ByteString
@@ -130,8 +136,8 @@ unfoldrE f b  =
         Right (a, new_b) -> let (a', b') = unfoldrE f new_b in (a : a', b')
         Left new_b       -> ([], new_b)
 
-showB :: (BS.Show a) => a -> B.ByteString
-showB = B.concat . BL.toChunks . BS.show
+showB :: (Show a) => a -> B.ByteString
+showB = B.pack . show
 
 readInt_ :: (Num a) => B.ByteString -> a
 readInt_ str =
@@ -239,3 +245,11 @@ deleteBy2 eq x (y:ys)    = if x `eq` y then ys else y : deleteBy2 eq x ys
 deleteFirstsBy2          :: (a -> b -> Bool) -> [a] -> [b] -> [a]
 deleteFirstsBy2 eq       =  foldl (flip (deleteBy2 (flip eq)))
 
+sanitizeName :: B.ByteString -> B.ByteString
+sanitizeName = B.map sc
+    where
+        sc c | isAlphaNum c = c
+             | otherwise = '_'
+
+isRegistered :: ClientInfo -> Bool
+isRegistered = (<) 0 . B.length . webPassword
