@@ -1,4 +1,4 @@
---------------------------------
+-------------------------------
 -- CTF_BLIZZARD 0.9
 --------------------------------
 
@@ -115,10 +115,10 @@ local ropeGear = nil
 local numhhs = 0 -- store number of hedgehogs
 local hhs = {} -- store hedgehog gears
 
-local numTeams --  store the number of teams in the game
 local teamNameArr = {}	-- store the list of teams
 local teamSize = {}	-- store how many hogs per team
 local teamIndex = {} -- at what point in the hhs{} does each team begin
+local clanTeams = {} -- list of teams per clan
 
 -------------------
 -- flag variables
@@ -134,8 +134,6 @@ local fSpawnY = {}		-- spawn Y for flags
 
 local fThiefX = {}
 local fThiefY = {}
-local FTTC = 0 -- flag thief tracker counter
---local fThiefsHealed = false
 
 local fSpawnC = {}
 local fCirc = {} -- flag/carrier marker circles
@@ -159,7 +157,6 @@ local vCircCol = {}
 
 local redTel
 local orangeTel
---local areaArr = {} -- no longer used
 
 local zXMin = {}
 local zWidth = {}
@@ -175,20 +172,19 @@ local zCount = 0
 
 function ManageTeleporterEffects()
 	effectTimer = effectTimer + 1
-	if effectTimer > 50 then -- 100
+	if effectTimer > 50 then
 		effectTimer = 0
 
 		for i = 0,1 do
-			eX = 10 + zXMin[i] + GetRandom(zWidth[i]-10)
-			eY = 50 + zYMin[i] + GetRandom(zHeight[i]-110)
+			local eX = 10 + zXMin[i] + GetRandom(zWidth[i]-10)
+			local eY = 50 + zYMin[i] + GetRandom(zHeight[i]-110)
 
-	-- steam and smoke and DUST look good, smokering looks trippy
-	-- smoketrace and eviltrace are not effected by wind?
-	-- chunk is a LR falling gear
-			tempE = AddVisualGear(eX, eY, vgtDust, 0, false)
+			-- steam and smoke and DUST look good, smokering looks trippy
+			-- smoketrace and eviltrace are not effected by wind?
+			-- chunk is a LR falling gear
+			local tempE = AddVisualGear(eX, eY, vgtDust, 0, false)
 			if tempE ~= 0 then
-				g1, g2, g3, g4, g5, g6, g7, g8, g9, g10 = GetVisualGearValues(tempE)
-				SetVisualGearValues(tempE, eX, eY, g3, g4, g5, g6, g7, g8, g9, fCol[i])
+				SetVisualGearValues(tempE, eX, eY, nil, nil, nil, nil, nil, nil, nil, fCol[i])
 			end
 		end
 	end
@@ -226,13 +222,12 @@ end
 
 function CheckScore(teamID)
 
+	local alt
 	if teamID == 0 then
 		alt = 1
-		winner = "Red"
 
 	elseif teamID == 1 then
 		alt = 0
-		winner = "Blue"
 	end
 
 	if fCaptures[teamID] == 3 then
@@ -242,7 +237,6 @@ function CheckScore(teamID)
 				SetHealth(hhs[i],0)
 			end
 		end
-		--ShowMission("GAME OVER!", "Victory for the " .. winner .. " Team!", "Hooray!", 0, 0)
 		ShowMission(loc("GAME OVER!"), loc("Victory for the ") .. GetHogTeamName(CurrentHedgehog), loc("Hooray!"), 0, 0)
 	end
 
@@ -253,7 +247,7 @@ function HandleRespawns()
 	for i = 0, 1 do
 
 		if fNeedsRespawn[i] == true then
-			fGear[i] = SpawnAmmoCrate(fSpawnX[i],fSpawnY[i],amSkip)
+			fGear[i] = SpawnFakeAmmoCrate(fSpawnX[i],fSpawnY[i],false,false)
 			--fGear[i] = SpawnHealthCrate(fSpawnX[i],fSpawnY[i])
 			fNeedsRespawn[i] = false
 			fIsMissing[i] = false -- new, this should solve problems of a respawned flag being "returned" when a player tries to score
@@ -266,6 +260,8 @@ end
 
 function FlagDeleted(gear)
 
+	local wtf, bbq
+	PlaySound(sndShotgunReload)
 	if (gear == fGear[0]) then
 		wtf = 0
 		bbq = 1
@@ -274,11 +270,8 @@ function FlagDeleted(gear)
 		bbq = 0
 	end
 
-	--ShowMission("OH HAI!", "FlagDeleted was called", "Oh noes!", -amBazooka, 0)
-
 	if CurrentHedgehog ~= nil then
 
-		--ShowMission("GUESS WAT?", "I'm not nil", "Oh noes!", -amBazooka, 0)
 		--if the player picks up the flag
 		if CheckDistance(CurrentHedgehog, fGear[wtf]) < 1600 then
 
@@ -290,13 +283,14 @@ function FlagDeleted(gear)
 				fNeedsRespawn[wtf] = true
 				fIsMissing[bbq] = false
 				fNeedsRespawn[bbq] = true
-				fCaptures[wtf] = fCaptures[wtf] +1					--fCaptures[wtf]
+				fCaptures[wtf] = fCaptures[wtf] +1
 
-				--ShowMission(LOC_NOT("You have SCORED!!"), "Red Team: " .. fCaptures[0], "Blue Team: " .. fCaptures[1], -amBazooka, 0)
-				ShowMission(loc("You have SCORED!!"), GetHogTeamName(CurrentHedgehog) .. ": " .. fCaptures[wtf], loc("Opposing Team: ") .. fCaptures[bbq], 0, 0)
+				AddCaption(string.format(loc("%s has scored!"), GetHogTeamName(CurrentHedgehog)), 0xFFFFFFFF, capgrpGameState)
+				for i=1, #clanTeams[wtf] do
+					SetTeamLabel(clanTeams[wtf][i], fCaptures[wtf])
+				end
 
-				PlaySound(sndVictory)
-				--SetEffect(fThief[bbq], hePoisoned, false)
+				PlaySound(sndHomerun)
 				fThief[bbq] = nil -- player no longer has the enemy flag
 				CheckScore(wtf)
 
@@ -310,19 +304,15 @@ function FlagDeleted(gear)
 					HandleRespawns() -- this will set fIsMissing[wtf] to false :)
 					AddCaption(loc("Flag returned!"))
 				elseif fIsMissing[wtf] == false then
-					AddCaption(loc("That was pointless.") .. loc("The flag will respawn next round."))
+					AddCaption(loc("That was pointless. The flag will respawn next round."))
 				end
 
-				--fIsMissing[wtf] = false
-				--ShowMission("Flag returned!", "Hooray", "", -amBazooka, 0)
-
 			--if the player is taking the enemy flag
 			elseif GetHogClan(CurrentHedgehog) == bbq then
 				fIsMissing[wtf] = true
 				for i = 0,numhhs-1 do
 					if CurrentHedgehog == hhs[i] then
 						fThief[wtf] = hhs[i]
-						--SetEffect(fThief[wtf], hePoisoned, true)
 					end
 				end
 
@@ -354,7 +344,6 @@ function FlagDeleted(gear)
 	-- probably only gets called if the flag thief drowns himself
 	-- otherwise the above one will work fine
 	else
-		--ShowMission("NIL PLAYER!", "Oh snap", "Oh noes!", -amBazooka, 0)
 		fGear[wtf] = nil
 		fIsMissing[wtf] = true
 		fNeedsRespawn[wtf] = true
@@ -365,6 +354,7 @@ end
 
 function FlagThiefDead(gear)
 
+	local wtf, bbq
 	if (gear == fThief[0]) then
 		wtf = 0
 		bbq = 1
@@ -376,9 +366,9 @@ function FlagThiefDead(gear)
 	if fThief[wtf] ~= nil then
 		
 		if fThiefY[wtf] > 2040 then
-			fGear[wtf] = SpawnAmmoCrate(fThiefX[wtf],(fThiefY[wtf]+10),amSkip)
+			fGear[wtf] = SpawnFakeAmmoCrate(fThiefX[wtf],(fThiefY[wtf]+10),false,false)
 		else
-			fGear[wtf] = SpawnAmmoCrate(fThiefX[wtf],(fThiefY[wtf]-50),amSkip)
+			fGear[wtf] = SpawnFakeAmmoCrate(fThiefX[wtf],(fThiefY[wtf]-50),false,false)
 		end
 
 		AddVisualGear(fThiefX[wtf], fThiefY[wtf], vgtBigExplosion, 0, false)
@@ -391,14 +381,11 @@ function HandleCircles()
 
 	for i = 0, 1 do
 		if fIsMissing[i] == false then -- draw a circle at the flag's spawning place
-			--SetVisualGearValues(fCirc[i], fSpawnX[i],fSpawnY[i], 20, 200, 0, 0, 100, 33, 2, fCol[i])
 			SetVisualGearValues(fCirc[i], fSpawnX[i],fSpawnY[i], vCircMinA[i], vCircMaxA[i], vCircType[i], vCircPulse[i], vCircFuckAll[i], vCircRadius[i], vCircWidth[i], vCircCol[i])
 		elseif (fIsMissing[i] == true) and (fNeedsRespawn[i] == false) then
 			if fThief[i] ~= nil then -- draw circle round flag carrier
-				--SetVisualGearValues(fCirc[i], fThiefX[i], fThiefY[i], 20, 200, 0, 0, 100, 33, 2, fCol[i])
 				SetVisualGearValues(fCirc[i], fThiefX[i], fThiefY[i], vCircMinA[i], vCircMaxA[i], vCircType[i], vCircPulse[i], vCircFuckAll[i], vCircRadius[i], vCircWidth[i], vCircCol[i])
 			elseif fThief[i] == nil then -- draw cirle round dropped flag
-				--SetVisualGearValues(fCirc[i], GetX(fGear[i]), GetY(fGear[i]), 20, 200, 0, 0, 100, 33, 2, fCol[i])
 				SetVisualGearValues(fCirc[i], GetX(fGear[i]),GetY(fGear[i]), vCircMinA[i], vCircMaxA[i], vCircType[i], vCircPulse[i], vCircFuckAll[i], vCircRadius[i], vCircWidth[i], vCircCol[i])
 			end
 		end
@@ -416,16 +403,12 @@ end
 
 function CheckDistance(gear1, gear2)
 
-	g1X, g1Y = GetGearPosition(gear1)
-	g2X, g2Y = GetGearPosition(gear2)
+	local g1X, g1Y = GetGearPosition(gear1)
+	local g2X, g2Y = GetGearPosition(gear2)
 
 	g1X = g1X - g2X
 	g1Y = g1Y - g2Y
-	z = (g1X*g1X) + (g1Y*g1Y)
-
-	--dist = math.sqrt(z)
-
-	dist = z
+	local dist = (g1X*g1X) + (g1Y*g1Y)
 
 	return dist
 
@@ -433,7 +416,7 @@ end
 
 function CheckTeleporters()
 
-	teleportActive = false
+	local teleportActive = false
 
 	if (GearIsInZone(CurrentHedgehog, redTel) == true) and (GetHogClan(CurrentHedgehog) == 0) then
 		teleportActive = true
@@ -452,15 +435,12 @@ function CheckTeleporters()
 					SetGearMessage(CurrentHedgehog, gmAttack)
 				end
 			end
-			--AddCaption(actionReset .. ";" .. "attack")
 		elseif actionReset == 10 then
 			SetGearMessage(CurrentHedgehog, 0)
-			--AddCaption(actionReset .. ";" .. "reset")
 		elseif actionReset == 20 then
 			AddVisualGear(GetX(CurrentHedgehog), GetY(CurrentHedgehog), vgtBigExplosion, 0, false)
 			SetGearPosition(CurrentHedgehog,destinationX,destinationY)
 			AddVisualGear(GetX(CurrentHedgehog), GetY(CurrentHedgehog), vgtBigExplosion, 0, false)
-			--AddCaption(actionReset .. ";" .. "teleport")
 		end
 
 		actionReset = actionReset + 1
@@ -477,42 +457,14 @@ function RebuildTeamInfo()
 
 	-- make a list of individual team names
 	for i = 0, (TeamsCount-1) do
-		teamNameArr[i] = i
+		teamNameArr[i] = GetTeamName(i)
 		teamSize[i] = 0
 		teamIndex[i] = 0
-	end
-	numTeams = 0
-
-	for i = 0, (numhhs-1) do
-
-		z = 0
-		unfinished = true
-		while(unfinished == true) do
-
-			newTeam = true
-			tempHogTeamName = GetHogTeamName(hhs[i]) -- this is the new name
-
-			if tempHogTeamName == teamNameArr[z] then
-				newTeam = false
-				unfinished = false
-			end
-
-			z = z + 1
-
-			if z == TeamsCount then
-				unfinished = false
-				if newTeam == true then
-					teamNameArr[numTeams] = tempHogTeamName
-					numTeams = numTeams + 1
-				end
-			end
-
-		end
-
+		SetTeamLabel(teamNameArr[i], "0")
 	end
 
 	-- find out how many hogs per team, and the index of the first hog in hhs
-	for i = 0, numTeams-1 do
+	for i = 0, TeamsCount-1 do
 
 		for z = 0, numhhs-1 do
 			if GetHogTeamName(hhs[z]) == teamNameArr[i] then
@@ -520,10 +472,17 @@ function RebuildTeamInfo()
 					teamIndex[i] = z -- should give starting index
 				end
 				teamSize[i] = teamSize[i] + 1
-				--add a pointer so this hog appears at i in hhs
+
+				local clan = GetHogClan(hhs[z])
+				-- Also remember the clan to which the team belongs to
+				if not clanTeams[clan] then
+					clanTeams[clan] = {}
+				end
+				table.insert(clanTeams[clan], teamNameArr[i])
 			end
 		end
 
+
 	end
 
 end
@@ -538,21 +497,21 @@ function HandleCrateDrops()
 
 		r = GetRandom(8)
 		if r == 0 then
-			SpawnUtilityCrate(0,0,amSwitch)
+			SpawnSupplyCrate(0,0,amSwitch)
 		elseif r == 1 then
-			SpawnUtilityCrate(0,0,amTeleport)
+			SpawnSupplyCrate(0,0,amTeleport)
 		elseif r == 2 then
-			SpawnUtilityCrate(0,0,amJetpack)
+			SpawnSupplyCrate(0,0,amJetpack)
 		elseif r == 3 then
-			SpawnUtilityCrate(0,0,amExtraTime)
+			SpawnSupplyCrate(0,0,amExtraTime)
 		elseif r == 4 then
-			SpawnUtilityCrate(0,0,amGirder)
+			SpawnSupplyCrate(0,0,amGirder)
 		elseif r == 5 then
-			SpawnAmmoCrate(0,0,amDynamite)
+			SpawnSupplyCrate(0,0,amDynamite)
 		elseif r == 6 then
-			SpawnAmmoCrate(0,0,amFlamethrower)
+			SpawnSupplyCrate(0,0,amFlamethrower)
 		elseif r == 7 then
-			SpawnUtilityCrate(0,0,amPortalGun)
+			SpawnSupplyCrate(0,0,amPortalGun)
 		end
 
 	end
@@ -567,23 +526,22 @@ function onGameInit()
 
 	-- Things we don't modify here will use their default values.
 	GameFlags = gfDivideTeams -- Game settings and rules
-	TurnTime = 30000 -- (was 30) The time the player has to move each round (in ms)
+	TurnTime = 30000 -- The time the player has to move each round (in ms)
 	CaseFreq = 0 -- The frequency of crate drops
 	MinesNum = 0 -- The number of mines being placed
-	MinesTime  = 2000
 	Explosives = 0 -- The number of explosives being placed
-	Delay = 10 -- The delay between each round
-	WaterRise = 0 -- I sure hope this works	
-	--SuddenDeathTurns = 99 -- suddendeath is off, effectively
+	-- Disable Sudden Death
+	WaterRise = 0
+	HealthDecrease = 0
 	Map = "Blizzard" -- The map to be played
-	Theme = "Snow" -- The theme to be used "Nature"
+	Theme = "Snow" -- The theme to be used
 
 end
 
 
 function onGameStart()
 
-	ShowMission("CTF_BLIZZARD", "", loc(" - Return the enemy flag to your base to score | - First team to 3 captures wins | - You may only score when your flag is in your base | - Hogs will drop the flag if killed, or drowned | - Dropped flags may be returned or recaptured | - Hogs respawn when killed"), 0, 0)
+	ShowMission(loc("CTF_Blizzard"), loc("Capture The Flag"), loc(" - Return the enemy flag to your base to score | - First team to 3 captures wins | - You may only score when your flag is in your base | - Hogs will drop the flag if killed, or drowned | - Dropped flags may be returned or recaptured | - Hogs respawn when killed"), 0, 0)
 
 
 	-- initialize teleporters
@@ -593,9 +551,8 @@ function onGameStart()
 
 	--new improved placement schematics aw yeah
 	RebuildTeamInfo()
-	--ShowMission("Team Info Rebuilt", "Here you go:", "TeamCount: " .. TeamsCount .. "|" .. teamNameArr[0] .. ": " .. teamSize[0] .. " Hogs|" .. teamNameArr[1] .. ": " .. teamSize[1] .. " Hogs|" .. teamNameArr[2] .. ": " .. teamSize[2] .. " Hogs|", 0, 0)
-	team1Placed = 0
-	team2Placed = 0
+	local team1Placed = 0
+	local team2Placed = 0
 	for i = 0, (TeamsCount-1) do
 		for g = teamIndex[i], (teamIndex[i]+teamSize[i]-1) do
 			if GetHogClan(hhs[g]) == 0 then
@@ -617,8 +574,7 @@ function onGameStart()
 
 
 	--spawn starting ufos and or super weapons
-	SpawnAmmoCrate(2048,1858,amJetpack)
-	--SpawnUtilityCrate(2048,1858,amExtraTime)
+	SpawnSupplyCrate(2048,1858,amJetpack)
 
 	--set flag spawn points and spawn the flags
 	fSpawnX[0] = 957
@@ -627,7 +583,7 @@ function onGameStart()
 	fSpawnY[1] = 1747
 
 	for i = 0, 1 do
-		fGear[i] = SpawnAmmoCrate(fSpawnX[i],fSpawnY[i],amSkip)
+		fGear[i] = SpawnFakeAmmoCrate(fSpawnX[i],fSpawnY[i],false,false)
 		fCirc[i] = AddVisualGear(fSpawnX[i],fSpawnY[i],vgtCircle,0,true)
 		fCol[i] = GetClanColor(i)
 
@@ -657,25 +613,9 @@ end
 
 function onNewTurn()
 
-	if lastTeam ~= GetHogTeamName(CurrentHedgehog) then
-		lastTeam = GetHogTeamName(CurrentHedgehog)
-	end
-
-	for i = 0, 1 do
-		if fThief[i] ~= nil then
-			--adjust = 5 + GetHealth(fThief[i])
-			--SetHealth(fThief[i], adjust)
-			--AddCaption('Helped out the flag poisoned flag thiefs')
-		end
-	end
-
-	--AddCaption("Handling respawns")
 	HandleRespawns()
 	HandleCrateDrops()
 
-	--myC = AddVisualGear(GetX(CurrentHedgehog),GetY(CurrentHedgehog),vgtCircle,0,true)
-	--SetVisualGearValues(myC, GetX(CurrentHedgehog),GetY(CurrentHedgehog), 20, 200, 0, 0, 100, 50, 3, GetClanColor(GetHogClan(CurrentHedgehog)))
-
 end
 
 function onGameTick()
@@ -683,21 +623,16 @@ function onGameTick()
 	-- onRessurect calls AFTER you have resurrected,
 	-- so keeping track of x,y a few milliseconds before
 	-- is useful
-	--FTTC = FTTC + 1
-	--if FTTC == 100 then
-	--	FTTC = 0
-		for i = 0,1 do
-			if fThief[i] ~= nil then
-				fThiefX[i] = GetX(fThief[i])
-				fThiefY[i] = GetY(fThief[i])
-			end
+
+	for i = 0,1 do
+		if fThief[i] ~= nil then
+			fThiefX[i] = GetX(fThief[i])
+			fThiefY[i] = GetY(fThief[i])
 		end
-	--end
+	end
 
 	-- things we wanna check often
 	if (CurrentHedgehog ~= nil) then
-		--AddCaption(GetX(CurrentHedgehog) .. "; " .. GetY(CurrentHedgehog))
-		--AddCaption("Checking Teleporters")
 		CheckTeleporters()
 	end
 
@@ -715,14 +650,11 @@ function onAmmoStoreInit()
 	SetAmmo(amGrenade,9,0,0,0)
 	SetAmmo(amClusterBomb,4,0,0,0)
 
-	--SetAmmo(amDEagle, 4, 0, 0, 0)
 	SetAmmo(amShotgun, 9, 0, 0, 0)
 	SetAmmo(amFlamethrower, 1, 0, 0, 1)
 
 	SetAmmo(amFirePunch, 9, 0, 0, 0)
 	SetAmmo(amBaseballBat, 2, 0, 0, 0)
-	--SetAmmo(amKamikaze, 2, 0, 0, 0)
-
 
 	SetAmmo(amDynamite,2,0,0,1)
 	SetAmmo(amSMine,4,0,0,0)
@@ -747,8 +679,6 @@ end
 
 function onGearResurrect(gear)
 
-	--AddCaption("A gear has been resurrected!")
-
 	-- mark the flag thief as dead if he needed a respawn
 	for i = 0,1 do
 		if gear == fThief[i] then
@@ -767,28 +697,25 @@ function onGearResurrect(gear)
 
 end
 
-function onGearDamage(gear, damage)
-
-	-- >_< damn, occurs too fast, before the hog has finished moving / updated his health
-	--if GetGearType(gear) == gtHedgehog then
-	--	if damage > GetHealth(gear) then
-	--		AddVisualGear(GetX(gear), GetY(gear), vgtExplosion, 0, false)
-	--	end
-	--end
-
-end
+local excessHogsWarning = false
 
 function onGearAdd(gear)
 
 	if GetGearType(gear) == gtHedgehog then
 
-		hhs[numhhs] = gear
-		numhhs = numhhs + 1
-		SetEffect(gear, heResurrectable, 1)
+		if GetHogClan(gear) > 1 then
+			DeleteGear(gear)
+			if not excessHogsWarning then
+				WriteLnToChat(loc("Only two clans allowed! Excess hedgehogs will be removed."))
+				excessHogsWarning = true
+			end
+		else
+			hhs[numhhs] = gear
+			numhhs = numhhs + 1
+			SetEffect(gear, heResurrectable, 1)
+		end
 
-	end
-
-	if GetGearType(gear) == gtRope then
+	elseif GetGearType(gear) == gtRope then
 		ropeGear = gear
 	end
 
