@@ -62,6 +62,10 @@ local TimeCounter = 0
 local gameWon = false
 local pointLimit = 300
 
+local missionName = loc("Control")
+local missionCaption = loc("Domination game")
+local missionHelp
+
 local vCirc = {}
 local vCircCount = 0
 
@@ -221,6 +225,13 @@ function AwardPoints()
 		end
 	end
 
+	-- Update team labels
+	for i = 0,(TeamsCount-1) do
+		if teamNameArr[i] ~= " " then
+			SetTeamLabel(teamNameArr[i], teamScore[teamClan[i]])
+		end
+	end
+
 end
 
 -----------------
@@ -269,6 +280,7 @@ function RebuildTeamInfo()
 
 	-- find out how many hogs per team, and the index of the first hog in hhs
 	for i = 0, (numTeams-1) do
+		SetTeamLabel(GetTeamName(i), "0")
 		for z = 0, (numhhs-1) do
 			if GetHogTeamName(hhs[z]) == teamNameArr[i] then
 				teamClan[i] = GetHogClan(hhs[z])				
@@ -305,10 +317,11 @@ end
 function onGameInit()
 
 	-- Things we don't modify here will use their default values.
-	--GameFlags = gfInfAttack + gfSolidLand -- Game settings and rules
 	
-	GameFlags = band(bor(GameFlags, gfInfAttack + gfSolidLand), bnot(gfKing + gfForts))
+	EnableGameFlags(gfInfAttack, gfSolidLand)
+	DisableGameFlags(gfKing, gfAISurvival)
 	WaterRise = 0
+	HealthDecrease = 0
 
 end
 
@@ -354,6 +367,9 @@ function onGameStart()
 	for i = 0, (numTeams-1) do
 		pointLimit = pointLimit - 25
 	end
+
+	missionHelp = loc("Control pillars to score points.") .. "|" ..
+		string.format(loc("Score goal: %d"), pointLimit)
 	
 	-- reposition hogs if they are on control points until they are not or sanity limit kicks in
 	reN = 0
@@ -369,11 +385,16 @@ function onGameStart()
 		--AddCaption(zz) -- number of times it took to work
 	end
 
-	ShowMission("CONTROL", 
-	"", 
-	loc("Control pillars to score points.") .. "|" .. 
-	loc("Goal") .. ": " .. pointLimit .. " " .. loc("points"), 0, 0)
+	for h=1, numhhs do
+		-- Tardis screws up the game too much, teams might not get killed correctly after victory
+		-- if a hog is still in time-travel.
+		-- This could be fixed, removing the Tardis is just a simple and lazy fix.
+		AddAmmo(hhs[h], amTardis, 0)
+		-- Resurrector is pointless, all hogs are already automatically resurrected.
+		AddAmmo(hhs[h], amResurrector, 0)
+	end
 
+	ShowMission(missionName, missionCaption, missionHelp, 0, 0)
 
 end
 
@@ -408,20 +429,6 @@ function onNewTurn()
 			TurnTimeLeft = 1
 		end
 
-		totalComment = ""		
-		for i = 0,(TeamsCount-1) do
-				if teamNameArr[i] ~= " " then				-- i
-					teamComment[i] = teamNameArr[i] .. ": " .. teamScore[teamClan[i]] .. " " .. loc("points") .. "|"
-					totalComment = totalComment .. teamComment[i]			
-				elseif teamNameArr[i] == " " then
-					teamComment[i] = "|"
-				end
-			end
-			
-			ShowMission("CONTROL", 
-			loc("Team Scores") .. ":", 
-			totalComment, 0, 1600)
-	
 	end
 
 end
