@@ -2,12 +2,8 @@
 
 HedgewarsScriptLoad("/Scripts/Locale.lua")
 
-local score = {[0] = 0, [1] = 0, [2] = 0, [3] = 0, [4] = 0, [5] = 0}
-
 local ball = nil
 
-local started = false
-
 function onGameInit()
 	GameFlags = gfSolidLand + gfInvulnerable + gfDivideTeams
 	TurnTime = 20000
@@ -15,12 +11,14 @@ function onGameInit()
 	MinesNum = 0
 	Explosives = 0
 	Delay = 500
-	SuddenDeathTurns = 99999 -- "disable" sudden death
+	-- Disable Sudden Death
+	WaterRise = 0
+	HealthDecrease = 0
 end
 
 function onGameStart()
-	ShowMission(loc("Hedgewars-Knockball"), loc("Not So Friendly Match"), loc("Bat balls at your enemies and|push them into the sea!"), -amBaseballBat, 0)
-	started = true
+	ShowMission(loc("Knockball"), loc("Not So Friendly Match"), loc("Bat balls at your enemies and|push them into the sea!"), -amBaseballBat, 0)
+	SetAmmoTexts(amBaseballBat, loc("Baseball Bat with Ball"), loc("Knockball weapon"), loc("Throw a baseball at your foes|and send them flying!") .. "|" .. loc("Attack: Throw ball"))
 end
 
 function onGameTick()
@@ -46,26 +44,8 @@ function onGearAdd(gear)
 end
 
 function onGearDelete(gear)
-	if not started then
-		return
-	end
 	if gear == ball then
 		ball = nil
-	elseif (GetGearType(gear) == gtHedgehog) and CurrentHedgehog ~= nil then
-		local clan = GetHogClan(CurrentHedgehog)
-		local s
-		if clan ~= nil then
-			if GetHogClan(CurrentHedgehog) ~= GetHogClan(gear) then
-				score[clan] = score[clan] + 1
-				s = string.format(loc("%s is out and Team %d|scored a point!| |Score:"), GetHogName(gear), clan + 1)
-			else
-				score[clan] = score[clan] - 1
-				s = string.format(loc("%s is out and Team %d|scored a penalty!| |Score:"), GetHogName(gear), clan + 1)
-			end
-			s = s .. " " .. score[0]
-			for i = 1, ClansCount - 1 do s = s .. " - " .. score[i] end
-			ShowMission(loc("Hedgewars-Knockball"), loc("Not So Friendly Match"), s, -amBaseballBat, 0)
-		end
 	end
 end
 
