@@ -1,3 +1,90 @@
+--[[
+A Classic Fairytale: First Blood
+
+= SUMMARY =
+Simple introduction to the most basic aspects of gameplay.
+Basically a story-driven mini-tutorial. Does not replace
+a real tutorial, however.
+
+= GOAL =
+To complete the various tasks the chief gives to the player.
+
+= FLOW CHART =
+This section explains how this mission is (roughly)
+*supposed* to work, with a step-to-step list.
+Use this to hunt down bugs or for testing.
+
+All other missions in ACF will have the same section.
+
+These symbols are used:
+- Event
+| Choice (only one of these can happen)
+> End of mission
+: Go to another event in the flow chart
+
+“TBS” stands for “Turn-Based Stragegy”. It is used when the game switches from
+heavily scripted events to the default turn-based stragegy gameplay. This
+is not used in the first mission, however.
+
+== Linear events ==
+This is the expected course of events in chronological order.
+
+- Introduction; movement (left/right/jump) instructions
+- Player moves to mushroom
+- Double jump instructions, move to flower
+- Collect first crate (rope)
+- Rope instructions
+- Parachute crate appears in the right pit
+- Player collects parachute
+- Instruct player to move to mole head
+| If player stopped on mole head:
+    - Cut scene
+    - Place girder to block off right pit
+    - Spawn rope crate in left pit
+    - Player must collect crate safely
+    - If hurt: Reset player to mole head
+| If player skipped the mole (e.g. by roping) and instead went down the pit left from the mole:
+    - Different cut scene
+    - Place girder to block off right pit
+    - Spawn rope crate in left pit
+    - Player must collect this crate
+- Shoryuken crate spawns
+- Player destroys all targets
+- Rope challenge
+    - Player chooses rope challenge difficulty
+    - Crates spawn, one-by-one, while player collects them
+    | If player collects all crates in time:
+        - Proceed
+    | If player fails to collect all crates in time:
+        : Rope challenge restarts
+- Deagle crate spawns
+- Player collects deagle crate
+- Deagle targets spawn
+- Deagle targets destroyd
+- Cannibal and lots of ammo crates appear
+| Player kills cannibal
+    > Victory
+| Player moves close to cannibal
+    - Many weapon crates with melee weapons spawn
+    - Cut scene
+    - Player kills cannibal
+    > Victory
+
+== Non-linear events ==
+These events can be triggered at (theoretically) any time and interrupt
+the normal flow. Obvious events like “all player hogs dead” are omitted here.
+
+- Player hog damages princess:
+    - Princess complains
+- Player hog damages chief
+    - Chief complains
+- Player hog, Princess or Chief dead:
+    > Game over
+- Player hurt itself without dying:
+    - Chief mocks player
+
+]]
+
 HedgewarsScriptLoad("/Scripts/Locale.lua")
 HedgewarsScriptLoad("/Scripts/Animate.lua")
 
@@ -31,7 +118,8 @@ targYdif1 = {1179, 1313, 1734, 1441, 1152, 1259}
 targetPosX = {{821, 866, 789}, {614, 656, 638}, {1238, 1237, 1200}}
 targetPosY = {{1342, 1347, 1326}, {1112, 1121, 1061}, {1152, 1111, 1111}}
 crateNum = {6, 8}
-
+rope2GirderX = 3245
+rope2GirderY = 1190
 
 stage = 1
 cratesCollected = 0
@@ -39,7 +127,7 @@ chalTries = 0
 targetsDestroyed = 0
 targsWave = 1
 tTime = -1
-difficulty = 0
+difficulty = 1
 
 cannibalVisible = false
 cannibalKilles = false
@@ -52,6 +140,7 @@ princessKilled = false
 rope1Taken = false
 paraTaken = false
 rope2Taken = false
+rope2InProgress = false
 punchTaken = false
 canKilled = false
 desertTaken = false
@@ -63,15 +152,15 @@ elderFace = "Left"
 
 goals = {
   [startDialogue] = {loc("First Blood"), loc("First Steps"), loc("Press [Left] or [Right] to move around, [Enter] to jump"), 1, 4000},
-  [onShroomAnim] = {loc("First Blood"), loc("A leap in a leap"), loc("Go on top of the flower"), 1, 4000},
+  [onShroomAnim] = {loc("First Blood"), loc("A leap in a leap"), loc("Go on top of the flower") .. "|" .. loc("Hint: Press [Esc] to review the mission texts."), 1, 7000},
   [onFlowerAnim] = {loc("First Blood"), loc("Hightime"), loc("Collect the crate on the right.|Hint: Select the rope, [Up] or [Down] to aim, [Space] to fire, directional keys to move.|Ropes can be fired again in the air!"), 1, 7000},
   [tookParaAnim] = {loc("First Blood"), loc("Omnivore"), loc("Get on the head of the mole"), 1, 4000},
   [onMoleHeadAnim] = {loc("First Blood"), loc("The Leap of Faith"), loc("Use the parachute ([Space] while in air) to get the next crate"), 1, 4000},
-  [tookRope2Anim] = {loc("First Blood"), loc("The Rising"), loc("Do the deed"), 1, 4000},
+  [tookRope2Anim] = {loc("First Blood"), loc("The Rising"), loc("Get that crate!"), 1, 4000},
   [tookPunchAnim] = {loc("First Blood"), loc("The Slaughter"), loc("Destroy the targets!|Hint: Select the Shoryuken and hit [Space]|P.S. You can use it mid-air."), 1, 5000},
   [challengeAnim] = {loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000},
   [challengeFailedAnim] = {loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000},
-  [challengeCompletedAnim] = {loc("First Blood"), loc("The Ultimate Weapon"), loc("Destroy the targets!|Hint: [Up], [Down] to aim, [Space] to shoot"), 1, 5000},
+  [challengeCompletedAnim] = {loc("First Blood"), loc("The Ultimate Weapon"), loc("Get that crate!"), 1, 5000},
   [beforeKillAnim] = {loc("First Blood"), loc("The First Blood"), loc("Kill the cannibal!"), 1, 5000},
   [closeCannim] = {loc("First Blood"), loc("The First Blood"), loc("KILL IT!"), 1, 5000}
 }
@@ -88,7 +177,7 @@ end
 
 function SkipDamageAnim(anim)
   SwitchHog(youngh)
-  SetInputMask(0xFFFFFFFF)
+  AnimSetInputMask(0xFFFFFFFF)
 end
 
 function SkipOnShroom()
@@ -111,7 +200,7 @@ function AnimationSetup()
   table.insert(elderDamagedAnim, {func = AnimWait, args = {elderh, 500}, skipFunc = Skipanim, skipArgs = elderDamagedAnim})
   table.insert(elderDamagedAnim, {func = AnimSay, args = {elderh, loc("Violence is not the answer to your problems!"), SAY_SAY, 3000}})
   table.insert(elderDamagedAnim, {func = AnimGearWait, args = {youngh, 500}})
-  
+
   AddSkipFunction(startDialogue, Skipanim, {startDialogue})
   table.insert(startDialogue, {func = AnimWait, args = {youngh, 3500}, skipFunc = Skipanim, skipArgs = startDialogue})
   table.insert(startDialogue, {func = AnimCaption, args = {youngh, loc("Once upon a time, on an island with great natural resources, lived two tribes in heated conflict..."),  5000}})
@@ -124,7 +213,7 @@ function AnimationSetup()
   table.insert(startDialogue, {func = AnimJump, args = {youngh, "long"}})
   table.insert(startDialogue, {func = AnimTurn, args = {princess, "Right"}})
   table.insert(startDialogue, {func = AnimSwitchHog, args = {youngh}})
-  table.insert(startDialogue, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("First Steps"), loc("Press [Left] or [Right] to move around, [Enter] to jump"), 1, 4000}}) 
+  table.insert(startDialogue, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("First Steps"), loc("Press [Left] or [Right] to move around, [Enter] to jump"), 1, 4000}})
 
   AddSkipFunction(onShroomAnim, SkipOnShroom, {onShroomAnim})
   table.insert(onShroomAnim, {func = AnimSay, args = {elderh, loc("I can see you have been training diligently."), SAY_SAY, 4000}, skipFunc = Skipanim, skipArgs = onShroomAnim})
@@ -134,29 +223,29 @@ function AnimationSetup()
   table.insert(onShroomAnim, {func = AnimTurn, args = {elderh, "Left"}})
   table.insert(onShroomAnim, {func = AnimSay, args = {princess, loc("He moves like an eagle in the sky."), SAY_THINK, 4000}})
   table.insert(onShroomAnim, {func = AnimSwitchHog, args = {youngh}})
-  table.insert(onShroomAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("A leap in a leap"), loc("Go on top of the flower"), 1, 4000}}) 
+  table.insert(onShroomAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("A leap in a leap"), loc("Go on top of the flower") .. "|" .. loc("Hint: Press [Esc] to review the mission texts."), 1, 7000}})
 
   AddSkipFunction(onFlowerAnim, Skipanim, {onFlowerAnim})
   table.insert(onFlowerAnim, {func = AnimSay, args = {elderh, loc("See that crate farther on the right?"), SAY_SAY, 4000}})
   table.insert(onFlowerAnim, {func = AnimSay, args = {elderh, loc("Swing, Leaks A Lot, on the wings of the wind!"), SAY_SAY, 6000}})
   table.insert(onFlowerAnim, {func = AnimSay, args = {princess, loc("His arms are so strong!"), SAY_THINK, 4000}})
   table.insert(onFlowerAnim, {func = AnimSwitchHog, args = {youngh}})
-  table.insert(onFlowerAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("Hightime"), loc("Collect the crate on the right.|Hint: Select the rope, [Up] or [Down] to aim, [Space] to fire, directional keys to move.|Ropes can be fired again in the air!"), 1, 7000}}) 
-  
+  table.insert(onFlowerAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("Hightime"), loc("Collect the crate on the right.|Hint: Select the rope, [Up] or [Down] to aim, [Space] to fire, directional keys to move.|Ropes can be fired again in the air!"), 1, 7000}})
+
   AddSkipFunction(tookParaAnim, Skipanim, {tookParaAnim})
   table.insert(tookParaAnim, {func = AnimGearWait, args = {youngh, 1000}, skipFunc = Skipanim, skipArgs = tookParaAnim})
   table.insert(tookParaAnim, {func = AnimSay, args = {elderh, loc("Use the rope to get on the head of the mole, young one!"), SAY_SHOUT, 4000}})
   table.insert(tookParaAnim, {func = AnimSay, args = {elderh, loc("Worry not, for it is a peaceful animal! There is no reason to be afraid..."), SAY_SHOUT, 5000}})
   table.insert(tookParaAnim, {func = AnimSay, args = {elderh, loc("We all know what happens when you get frightened..."), SAY_SAY, 4000}})
   table.insert(tookParaAnim, {func = AnimSay, args = {youngh, loc("So humiliating..."), SAY_SAY, 4000}})
-  table.insert(tookParaAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("Omnivore"), loc("Get on the head of the mole"), 1, 4000}}) 
+  table.insert(tookParaAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("Omnivore"), loc("Get on the head of the mole"), 1, 4000}})
   table.insert(tookParaAnim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(onMoleHeadAnim, Skipanim, {onMoleHeadAnim})
   table.insert(onMoleHeadAnim, {func = AnimSay, args = {elderh, loc("Perfect! Now try to get the next crate without hurting yourself!"), SAY_SAY, 4000}, skipFunc = Skipanim, skipArgs = onMoleHeadAnim})
   table.insert(onMoleHeadAnim, {func = AnimSay, args = {elderh, loc("The giant umbrella from the last crate should help break the fall."), SAY_SAY, 4000}})
   table.insert(onMoleHeadAnim, {func = AnimSay, args = {princess, loc("He's so brave..."), SAY_THINK, 4000}})
-  table.insert(onMoleHeadAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Leap of Faith"), loc("Use the parachute ([Space] while in air) to get the next crate"), 1, 4000}}) 
+  table.insert(onMoleHeadAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Leap of Faith"), loc("Use the parachute ([Space] while in air) to get the next crate"), 1, 4000}})
   table.insert(onMoleHeadAnim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(pastMoleHeadAnim, Skipanim, {pastMoleHeadAnim})
@@ -167,13 +256,13 @@ function AnimationSetup()
   AddSkipFunction(tookRope2Anim, Skipanim, {tookRope2Anim})
   table.insert(tookRope2Anim, {func = AnimSay, args = {elderh, loc("Impressive...you are still dry as the corpse of a hawk after a week in the desert..."), SAY_SAY, 5000}, skipFunc = Skipanim, skipArgs = tookRope2Anim})
   table.insert(tookRope2Anim, {func = AnimSay, args = {elderh, loc("You probably know what to do next..."), SAY_SAY, 4000}})
-  table.insert(tookRope2Anim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Rising"), loc("Do the deed"), 1, 4000}}) 
+  table.insert(tookRope2Anim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Rising"), loc("Get that crate!"), 1, 4000}})
   table.insert(tookRope2Anim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(tookPunchAnim, Skipanim, {tookPunchAnim})
   table.insert(tookPunchAnim, {func = AnimSay, args = {elderh, loc("It is time to practice your fighting skills."), SAY_SAY, 4000}})
   table.insert(tookPunchAnim, {func = AnimSay, args = {elderh, loc("Imagine those targets are the wolves that killed your parents! Take your anger out on them!"), SAY_SAY, 5000}})
-  table.insert(tookPunchAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Slaughter"), loc("Destroy the targets!|Hint: Select the Shoryuken and hit [Space]|P.S. You can use it mid-air."), 1, 5000}}) 
+  table.insert(tookPunchAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Slaughter"), loc("Destroy the targets!|Hint: Select the Shoryuken and hit [Space]|P.S. You can use it mid-air."), 1, 5000}})
   table.insert(tookPunchAnim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(challengeAnim, Skipanim, {challengeAnim})
@@ -186,7 +275,7 @@ function AnimationSetup()
 
   AddSkipFunction(challengeFailedAnim, Skipanim, {challengeFailedAnim})
   table.insert(challengeFailedAnim, {func = AnimSay, args = {elderh, loc("Hmmm...perhaps a little more time will help."), SAY_SAY, 4000}, skipFunc = Skipanim, skipArgs = challengeFailedAnim})
-  table.insert(challengeFailedAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000}}) 
+  table.insert(challengeFailedAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000}})
   table.insert(challengeFailedAnim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(challengeCompletedAnim, Skipanim, {challengeCompletedAnim})
@@ -194,25 +283,26 @@ function AnimationSetup()
   table.insert(challengeCompletedAnim, {func = AnimSay, args = {elderh, loc("You have proven yourself worthy to see our most ancient secret!"), SAY_SAY, 4000}})
   table.insert(challengeCompletedAnim, {func = AnimSay, args = {elderh, loc("The weapon in that last crate was bestowed upon us by the ancients!"), SAY_SAY, 4000}})
   table.insert(challengeCompletedAnim, {func = AnimSay, args = {elderh, loc("Use it with precaution!"), SAY_SAY, 4000}})
-  table.insert(challengeCompletedAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Ultimate Weapon"), loc("Destroy the targets!|Hint: [Up], [Down] to aim, [Space] to shoot"), 1, 5000}}) 
+  table.insert(challengeCompletedAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The Ultimate Weapon"), loc("Get that crate!"), 1, 5000}})
   table.insert(challengeCompletedAnim, {func = AnimSwitchHog, args = {youngh}})
 
   AddSkipFunction(beforeKillAnim, Skipanim, {beforeKillAnim})
+  table.insert(beforeKillAnim, {func = AnimWait, args = {elderh, 100}})
   table.insert(beforeKillAnim, {func = AnimSay, args = {elderh, loc("What do my faulty eyes observe? A spy!"), SAY_SHOUT, 4000}, skipFunc = Skipanim, skipArgs = beforeKillAnim})
   table.insert(beforeKillAnim, {func = AnimFollowGear, args = {cannibal}})
   table.insert(beforeKillAnim, {func = AnimWait, args = {cannibal, 1000}})
   table.insert(beforeKillAnim, {func = AnimSay, args = {elderh, loc("Destroy him, Leaks A Lot! He is responsible for the deaths of many of us!"), SAY_SHOUT, 4000}})
   table.insert(beforeKillAnim, {func = AnimSay, args = {cannibal, loc("Oh, my!"), SAY_THINK, 4000}})
-  table.insert(beforeKillAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The First Blood"), loc("Kill the cannibal!"), 1, 5000}}) 
+  table.insert(beforeKillAnim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The First Blood"), loc("Kill the cannibal!"), 1, 5000}})
   table.insert(beforeKillAnim, {func = AnimSwitchHog, args = {youngh}})
-  
+
   AddSkipFunction(closeCannim, Skipanim, {closeCannim})
   table.insert(closeCannim, {func = AnimSay, args = {elderh, loc("I see you would like his punishment to be more...personal..."), SAY_SAY, 4000}, skipFunc = Skipanim, skipArgs = closeCannim})
   table.insert(closeCannim, {func = AnimSay, args = {cannibal, loc("I'm certain that this is a misunderstanding, fellow hedgehogs!"), SAY_SAY, 4000}})
   table.insert(closeCannim, {func = AnimSay, args = {cannibal, loc("If only I were given a chance to explain my being here..."), SAY_SAY, 4000}})
   table.insert(closeCannim, {func = AnimSay, args = {elderh, loc("Do not let his words fool you, young one! He will stab you in the back as soon as you turn away!"), SAY_SAY, 6000}})
   table.insert(closeCannim, {func = AnimSay, args = {elderh, loc("Here...pick your weapon!"), SAY_SAY, 5000}})
-  table.insert(closeCannim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The First Blood"), loc("KILL IT!"), 1, 5000}}) 
+  table.insert(closeCannim, {func = AnimShowMission, args = {youngh, loc("First Blood"), loc("The First Blood"), loc("KILL IT!"), 1, 5000}})
   table.insert(closeCannim, {func = AnimSwitchHog, args = {youngh}})
 
   table.insert(cannKilledAnim, {func = AnimSay, args = {elderh, loc("Yes, yeees! You are now ready to enter the real world!"), SAY_SHOUT, 6000}})
@@ -245,7 +335,7 @@ function CheckNeedToTurn(gear)
 end
 
 function DoNeedToTurn(gear)
-  if gear == princess then
+  if gear == princess and not princessDamaged then
     if GetX(princess) > GetX(youngh) then
       HogTurnLeft(princess, true)
       princessFace = "Left"
@@ -253,7 +343,7 @@ function DoNeedToTurn(gear)
       HogTurnLeft(princess, false)
       princessFace = "Right"
     end
-  else
+  elseif gear == elderh and not elderDamaged then
     if GetX(elderh) > GetX(youngh) then
       HogTurnLeft(elderh, true)
       elderFace = "Left"
@@ -265,11 +355,14 @@ function DoNeedToTurn(gear)
 end
 
 function CheckDamage()
-  return youngdamaged and StoppedGear(youngh) 
+  return youngdamaged and StoppedGear(youngh)
 end
 
 function DoOnDamage()
   AddAnim(damageAnim)
+  if rope2InProgress and not rope2Taken then
+    AnimSetGearPosition(youngh, 3040, 1221)
+  end
   youngdamaged = false
   AddFunction({func = ResetTurnTime, args = {}})
 end
@@ -325,7 +418,7 @@ function CheckOnShroom()
 end
 
 function DoOnShroom()
-  ropeCrate1 = SpawnUtilityCrate(2751, 1194, amRope)
+  ropeCrate1 = SpawnSupplyCrate(2751, 1194, amRope, 100)
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(onShroomAnim)
   AddEvent(CheckOnFlower, {}, DoOnFlower, {}, 0)
@@ -337,7 +430,7 @@ end
 
 function DoOnFlower()
   AddAmmo(youngh, amRope, 100)
-  paraCrate = SpawnUtilityCrate(3245, 1758, amParachute)
+  paraCrate = SpawnSupplyCrate(3245, 1758, amParachute, 100)
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(onFlowerAnim)
   AddEvent(CheckTookParaCrate, {}, DoTookParaCrate, {}, 0)
@@ -363,23 +456,31 @@ end
 function CheckPastMoleHead()
   x = GetX(youngh)
   y = GetY(youngh)
-  return x < 3005 and y > 1500 and StoppedGear(youngh) 
+  return x < 3005 and y > 1500 and StoppedGear(youngh)
 end
 
 function DoPastMoleHead()
+  -- Initiate parachute challenge
   RemoveEventFunc(CheckOnMoleHead)
-  ropeCrate2 = SpawnUtilityCrate(2782, 1720, amRope)
+  ropeCrate2 = SpawnSupplyCrate(2782, 1720, amRope, 100)
+  rope2InProgress = true
   AddAmmo(youngh, amRope, 0)
   SetGearMessage(CurrentHedgehog, 0)
+  -- Block the way to the hole to the right, since the player loses the rope for this section
+  PlaceGirder(rope2GirderX, rope2GirderY, 6)
   AddAnim(pastMoleHeadAnim)
   AddEvent(CheckTookRope2, {}, DoTookRope2, {}, 0)
 end
 
 function DoOnMoleHead()
+  -- Initiate parachute challenge
   RemoveEventFunc(CheckPastMoleHead)
-  ropeCrate2 = SpawnUtilityCrate(2782, 1720, amRope)
+  ropeCrate2 = SpawnSupplyCrate(2782, 1720, amRope, 100)
+  rope2InProgress = true
   AddAmmo(youngh, amRope, 0)
   SetGearMessage(CurrentHedgehog, 0)
+  -- Block the way to the hole to the right, since the player loses the rope for this section
+  PlaceGirder(rope2GirderX, rope2GirderY, 6)
   AddAnim(onMoleHeadAnim)
   AddEvent(CheckTookRope2, {}, DoTookRope2, {}, 0)
 end
@@ -392,7 +493,7 @@ function DoTookRope2()
   AddAmmo(youngh, amRope, 100)
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(tookRope2Anim)
-  punchCrate = SpawnAmmoCrate(2460, 1321, amFirePunch)
+  punchCrate = SpawnSupplyCrate(2460, 1321, amFirePunch, 100)
   AddEvent(CheckTookPunch, {}, DoTookPunch, {})
 end
 
@@ -418,24 +519,24 @@ function CheckTargDestroyed()
 end
 
 function DoTargDestroyed()
+  AddAmmo(youngh, amFirePunch, 0)
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(challengeAnim)
   targetsDestroyed = 0
   AddFunction({func = SetChoice, args = {}})
-  ropeCrate3 = SpawnAmmoCrate(2000, 1200, amRope)
+  ropeCrate3 = SpawnSupplyCrate(2000, 1200, amRope, 100)
   AddEvent(CheckTookRope3, {}, AddAmmo, {youngh, amRope, 100}, 0)
   AddEvent(CheckCratesColled, {}, DoCratesColled, {}, 0)
   AddEvent(CheckChallengeWon, {}, DoChallengeWon, {}, 0)
   AddEvent(CheckTimesUp, {}, DoTimesUp, {}, 1)
-end
-
-function CheckChoice()
-  return difficulty ~= 0
+  -- Remove up the old mole blockade from the parachute challenge
+  EraseSprite(rope2GirderX, rope2GirderY, sprAmGirder, 6)
 end
 
 function DoChoice()
+  PlaySound(sndPlaced)
   difficultyChoice = false
-  SetInputMask(0xFFFFFFFF)
+  AnimSetInputMask(0xFFFFFFFF)
   StartChallenge(120000 + chalTries * 20000)
 end
 
@@ -454,7 +555,7 @@ function CheckChallengeWon()
 end
 
 function DoChallengeWon()
-  desertCrate = SpawnAmmoCrate(1240, 1212, amDEagle)
+  desertCrate = SpawnSupplyCrate(1240, 1212, amDEagle, 100)
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(challengeCompletedAnim)
   AddEvent(CheckDesertColled, {}, DoDesertColled, {}, 0)
@@ -473,7 +574,7 @@ function DoTimesUp()
   deleteCrate = true
   DeleteGear(crates[1])
   TurnTimeLeft = -1
-  AddCaption(loc("And so happenned that Leaks A Lot failed to complete the challenge! He landed, pressured by shame..."))
+  AddCaption(loc("And so happened that Leaks A Lot failed to complete the challenge! He landed, pressured by shame ..."))
   AddEvent(CheckChallengeFailed, {}, DoChallengeFailed, {}, 0)
 end
 
@@ -499,7 +600,7 @@ function DoDesertColled()
   PutTargets(1)
   AddEvent(CheckTargetsKilled, {}, DoTargetsKilled, {}, 1)
   AddEvent(CheckCannibalKilled, {}, DoCannibalKilledEarly, {}, 0)
-  ShowMission(loc("First Blood"), loc("The Bull's Eye"), loc("[Up], [Down] to aim, [Space] to shoot!"), 1, 5000)
+  ShowMission(loc("First Blood"), loc("The Bull's Eye"), loc("Destroy the targets!|Hint: [Up], [Down] to aim, [Space] to shoot"), 1, 5000)
 end
 
 function CheckTargetsKilled()
@@ -511,7 +612,7 @@ function DoTargetsKilled()
   targsWave = targsWave + 1
   if targsWave > 3 then
     RemoveEventFunc(CheckTargetsKilled)
-    SetState(cannibal, gstVisible)
+    RestoreHog(cannibal)
     cannibalVisible = true
     SetGearMessage(CurrentHedgehog, 0)
     AddAnim(beforeKillAnim)
@@ -532,9 +633,9 @@ end
 function DoCloseToCannibal()
   SetGearMessage(CurrentHedgehog, 0)
   AddAnim(closeCannim)
-  AddFunction({func = SpawnAmmoCrate, args = {targetPosX[1][1], targetPosY[1][1], amWhip}})
-  AddFunction({func = SpawnAmmoCrate, args = {targetPosX[1][2], targetPosY[1][2], amBaseballBat}})
-  AddFunction({func = SpawnAmmoCrate, args = {targetPosX[1][3], targetPosY[1][3], amHammer}})
+  AddFunction({func = SpawnSupplyCrate, args = {targetPosX[1][1], targetPosY[1][1], amWhip}})
+  AddFunction({func = SpawnSupplyCrate, args = {targetPosX[1][2], targetPosY[1][2], amBaseballBat}})
+  AddFunction({func = SpawnSupplyCrate, args = {targetPosX[1][3], targetPosY[1][3], amHammer}})
 end
 
 function CheckCannibalKilled()
@@ -558,14 +659,28 @@ function StartChallenge(time)
   cratesCollected = 0
   PutCrate(1)
   TurnTimeLeft = time
-  ShowMission(loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000) 
+  ShowMission(loc("First Blood"), loc("The Crate Frenzy"), loc("Collect the crates within the time limit!|If you fail, you'll have to try again."), 1, 5000)
+end
+
+function ChoiceDialog()
+  local dstr
+  if difficulty == 2 then
+    dstr = loc("Difficulty: Hard")
+  else
+    dstr = loc("Difficulty: Easy")
+  end
+  ShowMission(loc("First Blood"), loc("The Torment"),
+    loc("Your next task is to collect some crates by using the rope!") .. "|" ..
+    loc("Press [Left] and [Right] to change the difficulty.") .. "| |" ..
+    dstr .. "| |" ..
+    loc("Press [Attack] to begin."),
+    0, 9999000)
 end
 
 function SetChoice()
-  SetInputMask(band(0xFFFFFFFF, bnot(gmAnimate+gmAttack+gmDown+gmHJump+gmLJump+gmSlot+gmSwitch+gmTimer+gmUp+gmWeapon)))
+  AnimSetInputMask(0)
   difficultyChoice = true
-  ShowMission(loc("First Blood"), loc("The Torment"), loc("Select difficulty: [Left] - easier or [Right] - harder"), 0, 4000)
-  AddEvent(CheckChoice, {}, DoChoice, {}, 0) 
+  ChoiceDialog()
 end
 
 function SetTime(time)
@@ -582,9 +697,9 @@ function PutCrate(i)
     return
   end
   if difficulty == 1 then
-    crates[1] = SpawnAmmoCrate(targXdif1[i], targYdif1[i], amRope)
+    crates[1] = SpawnFakeAmmoCrate(targXdif1[i], targYdif1[i], false, false)
   else
-    crates[1] = SpawnAmmoCrate(targXdif2[i], targYdif2[i], amRope)
+    crates[1] = SpawnFakeAmmoCrate(targXdif2[i], targYdif2[i], false, false)
   end
 end
 
@@ -602,14 +717,14 @@ end
 -----------------------------Main Functions----------------------------
 
 function onGameInit()
-	Seed = 69 
-	GameFlags = gfInfAttack + gfSolidLand + gfDisableWind 
-	TurnTime = 100000 
+	Seed = 69
+	GameFlags = gfInfAttack + gfSolidLand + gfDisableWind
+	TurnTime = 100000
 	CaseFreq = 0
 	MinesNum = 0
 	MinesTime = 3000
 	Explosives = 0
-	Delay = 10 
+	Delay = 10
 	Map = "A_Classic_Fairytale_first_blood"
 	Theme = "Nature"
 
@@ -625,11 +740,11 @@ function onGameInit()
   AnimSetGearPosition(youngh, 1862, 1362)
   HogTurnLeft(youngh, false)
 
-  AddTeam(loc("Cannibals"), 14483456, "Skull", "Island", "Pirate","cm_vampire")
+  AddTeam(loc("Cannibals"), 14483456, "skull", "Island", "Pirate","cm_vampire")
   cannibal = AddHog(loc("Brainiac"), 0, 5, "Zombi")
   AnimSetGearPosition(cannibal, 525, 1256)
   HogTurnLeft(cannibal, false)
-  
+
   AnimInit()
   AnimationSetup()
 end
@@ -639,7 +754,7 @@ function onGameStart()
   TurnTimeLeft = -1
   FollowGear(youngh)
 	ShowMission(loc("A Classic Fairytale"), loc("First Blood"), loc("Finish your training|Hint: Animations can be skipped with the [Precise] key."), -amSkip, 0)
-  SetState(cannibal, gstInvisible)
+  HideHog(cannibal)
 
   AddAnim(startDialogue)
   princessFace = "Right"
@@ -661,6 +776,18 @@ function onGameTick()
   CheckEvents()
 end
 
+local choiceDialogTimer = 0
+function onGameTick20()
+  -- Make sure the choice dialog never disappears while it is active
+  if difficultyChoice then
+    choiceDialogTimer = choiceDialogTimer + 20
+    if choiceDialogTimer > 9990000 then
+      ChoiceDialog()
+      choiceDialogTimer = 0
+    end
+  end
+end
+
 function onGearDelete(gear)
   if gear == ropeCrate1 then
     rope1Taken = true
@@ -668,14 +795,23 @@ function onGearDelete(gear)
     paraTaken = true
   elseif gear == ropeCrate2 then
     rope2Taken = true
+    rope2InProgress = false
   elseif gear == ropeCrate3 then
     rope3Taken = true
-  elseif gear == crates[1] and deleteCrate == true then
-    deleteCrate = false
-  elseif gear == crates[1] and challengeFailed == false then
-    crates[1] = nil
-    cratesCollected = cratesCollected + 1
-    PutCrate(cratesCollected + 1)
+  elseif gear == crates[1] then
+    -- Play sound if challenge crate (fake crate) collected
+    if band(GetGearMessage(gear), gmDestroy) ~= 0 then
+      PlaySound(sndShotgunReload)
+    end
+
+    -- Update crate challenge
+    if deleteCrate == true then
+      deleteCrate = false
+    elseif challengeFailed == false then
+      crates[1] = nil
+      cratesCollected = cratesCollected + 1
+      PutCrate(cratesCollected + 1)
+    end
   elseif gear == punchCrate then
     punchTaken = true
   elseif gear == desertCrate then
@@ -686,7 +822,7 @@ function onGearDelete(gear)
       i = i + 1
     end
     targets[i] = nil
-    targetsDestroyed = targetsDestroyed + 1 
+    targetsDestroyed = targetsDestroyed + 1
   elseif gear == cannibal then
     cannibalKilled = true
   elseif gear == princess then
@@ -706,7 +842,7 @@ end
 
 function onNewTurn()
   if CurrentHedgehog == cannibal and cannibalVisible == false then
-    SetState(cannibal, gstInvisible)
+    RestoreHog(cannibal)
   end
   SwitchHog(youngh)
   FollowGear(youngh)
@@ -726,7 +862,6 @@ function onGearDamage(gear, damage)
   elseif gear == cannibal then
     cannibalVisible = true
     cannibalDamaged = true
-    SetState(cannibal, 0)
   end
 end
 
@@ -738,13 +873,19 @@ end
 
 function onLeft()
   if difficultyChoice == true then
-    difficulty = 1
+    if difficulty ~= 1 then
+       difficulty = 1
+    else
+       difficulty = 2
+    end
+    PlaySound(sndSwitchHog)
+    ChoiceDialog()
   end
 end
+onRight = onLeft
 
-function onRight()
+function onAttack()
   if difficultyChoice == true then
-    difficulty = 2
+    DoChoice()
   end
 end
-
