@@ -23,6 +23,7 @@ HedgewarsScriptLoad("/Missions/Campaign/A_Space_Adventure/global_functions.lua")
 -- globals
 local missionName = loc("Bad timing")
 local chooseToBattle = false
+local awaitingInput = false
 local previousHog = 0
 local heroPlayedFirstTurn = false
 local startBattleCalled = false
@@ -30,11 +31,19 @@ local startBattleCalled = false
 local dialog01 = {}
 local dialog02 = {}
 local dialog03 = {}
+
 -- mission objectives
 local goals = {
-	[dialog01] = {missionName, loc("Ready for Battle?"), loc("Walk left if you want to join Captain Lime or right if you want to decline his offer"), 1, 4000},
-	[dialog02] = {missionName, loc("Battle Starts Now!"), loc("You have chosen to fight! Lead the Green Bananas to battle and eliminate all the enemies"), 1, 4000},
-	[dialog03] = {missionName, loc("Time to run!"), loc("You have chosen to flee... Unfortunately the only place where you can launch your saucer is the left-most place on the map"), 1, 4000},
+	[dialog01] = {missionName, loc("Ready for Battle?"),
+		loc("Captain Lime offered his help if you assist him in battle.").."|"..
+		loc("What do you want to do?").."| |"..
+		loc("Fight: Press [Attack]").."|"..
+		loc("Flee: Press [Jump]"), 1, 9999000},
+	[dialog02] = {missionName, loc("Battle Starts Now!"), loc("You have chosen to fight!").."|"..loc("Lead the Green Bananas to battle and eliminate all the enemies!"), 1, 5000},
+	[dialog03] = {missionName, loc("Time to run!"), loc("You have chosen to flee.").."|"..loc("You have to reach the left-most place on the map."), 1, 5000},
+	["fight"] = {missionName, loc("Ready for Battle?"), loc("You have chosen to fight!"), 1, 2000},
+	["flee"] = {missionName, loc("Ready for Battle?"), loc("You have chosen to flee."), 1, 2000},
+	["flee_final"] = {missionName, loc("Time to run!"), loc("Knock off the enemies from the left-most place of the map!") .. "|" .. loc("Stay there to flee!"), 1, 6000},
 }
 -- crates
 local crateWMX = 2170
@@ -88,13 +97,13 @@ local yellowArmy = {
 	{name = loc("Naranja Jed"), x = 960 , y = 516, health = 40},
 }
 teamA.name = loc("Hog Solo")
-teamA.color = tonumber("38D61C",16) -- green
+teamA.color = 0x38D61C -- green
 teamB.name = loc("Green Bananas")
-teamB.color = tonumber("38D61C",16) -- green
+teamB.color = 0x38D61C -- green
 teamC.name = loc("Yellow Watermelons")
-teamC.color = tonumber("DDFF00",16) -- yellow
+teamC.color = 0xFFFF01 -- yellow
 teamD.name = loc("Captain Lime")
-teamD.color = tonumber("38D61C",16) -- green
+teamD.color = 0x38D61C -- green
 
 function onGameInit()
 	Seed = 1
@@ -104,25 +113,24 @@ function onGameInit()
 	MinesTime = 1
 	Explosives = 0
 	Delay = 3
-	SuddenDeathTurns = 100
+	-- Disable Sudden Death
+	HealthDecrease = 0
+	WaterRise = 0
 	HealthCaseAmount = 50
 	Map = "fruit01_map"
 	Theme = "Fruit"
 
 	-- Hog Solo
-	AddTeam(teamA.name, teamA.color, "Bone", "Island", "HillBilly", "cm_birdy")
+	AddTeam(teamA.name, teamA.color, "Simple", "Island", "Default", "hedgewars")
 	hero.gear = AddHog(hero.name, 0, 100, "war_desertgrenadier1")
 	AnimSetGearPosition(hero.gear, hero.x, hero.y)
 	HogTurnLeft(hero.gear, true)
 	-- Captain Lime
-	AddTeam(teamD.name, teamD.color, "Bone", "Island", "HillBilly", "cm_birdy")
-	green1.bot = AddHog(green1.name, 1, 200, "war_desertofficer")
-	AnimSetGearPosition(green1.bot, green1.x, green1.y)
-	green1.human =  AddHog(green1.name, 0, 200, "war_desertofficer")
-	AnimSetGearPosition(green1.human, green1.x, green1.y)
-	green1.gear = green1.human
+	AddTeam(teamD.name, teamD.color, "Cherry", "Island", "Default", "congo-brazzaville")
+	green1.gear = AddHog(green1.name, 0, 200, "war_desertofficer")
+	AnimSetGearPosition(green1.gear, green1.x, green1.y)
 	-- Green Bananas
-	AddTeam(teamB.name, teamB.color, "Bone", "Island", "HillBilly", "cm_birdy")
+	AddTeam(teamB.name, teamB.color, "Cherry", "Island", "Default", "congo-brazzaville")
 	green2.gear = AddHog(green2.name, 0, 100, "war_britmedic")
 	AnimSetGearPosition(green2.gear, green2.x, green2.y)
 	HogTurnLeft(green2.gear, true)
@@ -136,7 +144,7 @@ function onGameInit()
 	AnimSetGearPosition(green5.gear, green5.x, green5.y)
 	HogTurnLeft(green5.gear, true)
 	-- Yellow Watermelons
-	AddTeam(teamC.name, teamC.color, "Bone", "Island", "HillBilly", "cm_birdy")
+	AddTeam(teamC.name, teamC.color, "Flower", "Island", "Default", "cm_mog")
 	yellow1.gear = AddHog(yellow1.name, 1, 100, "war_desertgrenadier2")
 	AnimSetGearPosition(yellow1.gear, yellow1.x, yellow1.y)
 	-- the rest of the Yellow Watermelons
@@ -148,16 +156,16 @@ function onGameInit()
 
 	initCheckpoint("fruit01")
 
-	AnimInit()
+	AnimInit(true)
 	AnimationSetup()
 end
 
 function onGameStart()
+	AnimSetInputMask(0)
 	AnimWait(hero.gear, 3000)
 	FollowGear(hero.gear)
 
 	AddEvent(onHeroDeath, {hero.gear}, heroDeath, {hero.gear}, 0)
-	AddEvent(onHeroSelect, {hero.gear}, heroSelect, {hero.gear}, 0)
 
 	-- Green team weapons
 	local greenArmy = { green1, green2 }
@@ -186,11 +194,10 @@ function onGameStart()
 	for i=3,7 do
 		HideHog(yellowArmy[i].gear)
 	end
-	HideHog(green1.bot)
 
 	-- crates
 	SpawnHealthCrate(health1X, health1Y)
-	SpawnAmmoCrate(crateWMX, crateWMY, amWatermelon)
+	SpawnSupplyCrate(crateWMX, crateWMY, amWatermelon)
 
 	AddAnim(dialog01)
 	SendHealthStatsOff()
@@ -198,17 +205,17 @@ end
 
 function onNewTurn()
 	if not heroPlayedFirstTurn and CurrentHedgehog ~= hero.gear and startBattleCalled then
-		TurnTimeLeft = 0
+		EndTurn(true)
 	elseif not heroPlayedFirstTurn and CurrentHedgehog == hero.gear and startBattleCalled then
 		heroPlayedFirstTurn = true
 	elseif not heroPlayedFirstTurn and CurrentHedgehog == green1.gear then
-		TurnTimeLeft = 0
+		EndTurn(true)
 	else
 		if chooseToBattle then
 			if CurrentHedgehog == green1.gear then
 				TotalRounds = TotalRounds - 2
 				AnimSwitchHog(previousHog)
-				TurnTimeLeft = 0
+				EndTurn(true)
 			end
 			previousHog = CurrentHedgehog
 		end
@@ -225,6 +232,18 @@ function onGameTick()
 	CheckEvents()
 end
 
+local choiceDialogTimer = 0
+function onGameTick20()
+  -- Make sure the choice dialog never disappears while it is active
+  if awaitingInput then
+    choiceDialogTimer = choiceDialogTimer + 20
+    if choiceDialogTimer > 9990000 then
+      ShowMission(unpack(goals[dialog01]))
+      choiceDialogTimer = 0
+    end
+  end
+end
+
 function onGearDelete(gear)
 	if gear == hero.gear then
 		hero.dead = true
@@ -296,28 +315,39 @@ function onBattleWin(gear)
 	return win
 end
 
-function onEscapeWin(gear)
-	local escape = false
+function isHeroOnLaunchPad()
 	if not hero.dead and GetX(hero.gear) < 170 and GetY(hero.gear) > 1980 and StoppedGear(hero.gear) then
-		escape = true
-		local yellowTeam = { yellow1, unpack(yellowArmy) }
-		for i=1,8 do
-			if not yellowTeam[i].hidden and GetHealth(yellowTeam[i].gear) and GetX(yellowTeam[i].gear) < 170 then
-				escape = false
-				break
-			end
-		end
-	end
-	return escape
-end
-
-function onHeroSelect(gear)
-	if GetX(hero.gear) ~= hero.x then
 		return true
 	end
 	return false
 end
 
+function isLaunchPadEmpty(gear)
+	local yellowTeam = { yellow1, unpack(yellowArmy) }
+	for i=1, #yellowArmy+1 do
+		if not yellowTeam[i].hidden and GetHealth(yellowTeam[i].gear) and GetX(yellowTeam[i].gear) < 170 then
+			return false
+		end
+	end
+	return true
+end
+
+function onHeroOnLaunchPadWithEnemies()
+	return isHeroOnLaunchPad() and not isLaunchPadEmpty()
+end
+
+function heroOnLaunchPadWithEnemies()
+	ShowMission(unpack(goals["flee_final"]))
+end
+
+function onEscapeWin(gear)
+	local escape = false
+	if isHeroOnLaunchPad() then
+		escape = isLaunchPadEmpty()
+	end
+	return escape
+end
+
 -------------- ACTIONS ------------------
 
 function heroDeath(gear)
@@ -333,34 +363,33 @@ function battleWin(gear)
 	saveVariables()
 	SendStat(siGameResult, loc("Green Bananas won!"))
 	SendStat(siCustomAchievement, loc("You have eliminated all visible enemy hedgehogs!"))
-	SendStat(siPlayerKills,'1',teamA.name)
-	SendStat(siPlayerKills,'1',teamB.name)
-	SendStat(siPlayerKills,'0',teamC.name)
+	sendSimpleTeamRankings({teamA.name, teamD.name, teamB.name, teamC.name})
 	EndGame()
 end
 
 function escapeWin(gear)
+	RemoveEventFunc(heroOnLaunchPadWithEnemies)
 	-- add stats
 	saveVariables()
 	SendStat(siGameResult, loc("Hog Solo escaped successfully!"))
 	SendStat(siCustomAchievement, loc("You have reached the take-off area successfully!"))
-	SendStat(siPlayerKills,'1',teamA.name)
-	SendStat(siPlayerKills,'0',teamB.name)
-	SendStat(siPlayerKills,'0',teamC.name)
+	sendSimpleTeamRankings({teamA.name, teamD.name, teamB.name, teamC.name})
 	EndGame()
 end
 
-function heroSelect(gear)
-	TurnTimeLeft = 0
+function heroSelect()
+	awaitingInput = false
 	FollowGear(hero.gear)
-	if GetX(hero.gear) < hero.x then
+	if chooseToBattle == true then
 		chooseToBattle = true
+		ShowMission(unpack(goals["fight"]))
 		AddEvent(onGreen1Death, {green1.gear}, green1Death, {green1.gear}, 0)
 		AddEvent(onBattleWin, {hero.gear}, battleWin, {hero.gear}, 0)
 		AddAnim(dialog02)
-	elseif GetX(hero.gear) > hero.x then
-		HogTurnLeft(hero.gear, true)
+	else
+		ShowMission(unpack(goals["flee"]))
 		AddAmmo(green1.gear, amSwitch, 100)
+		AddEvent(onHeroOnLaunchPadWithEnemies, {hero.gear}, heroOnLaunchPadWithEnemies, {hero.gear}, 0)
 		AddEvent(onEscapeWin, {hero.gear}, escapeWin, {hero.gear}, 0)
 		local greenTeam = { green2, green3, green4, green5 }
 		for i=1,4 do
@@ -372,64 +401,70 @@ end
 
 -------------- ANIMATIONS ------------------
 
+function AfterDialog01()
+	AnimSwitchHog(hero.gear)
+	awaitingInput = true
+end
+
 function Skipanim(anim)
 	if goals[anim] ~= nil then
 		ShowMission(unpack(goals[anim]))
-    end
-    if anim == dialog01 then
-		AnimSwitchHog(hero.gear)
+	end
+	if anim == dialog01 then
+		AfterDialog01()
 	elseif anim == dialog02 or anim == dialog03 then
 		startBattle()
-    end
+	end
 end
 
 function AnimationSetup()
 	-- DIALOG 01 - Start, Captain Lime talks explains to Hog Solo
 	AddSkipFunction(dialog01, Skipanim, {dialog01})
-	table.insert(dialog01, {func = AnimWait, args = {hero.gear, 3000}})
-	table.insert(dialog01, {func = AnimCaption, args = {hero.gear, loc("Somewhere on the Planet of Fruits a terrible war is about to begin..."), 5000}})
+	table.insert(dialog01, {func = AnimWait, args = {hero.gear, 1000}})
+	table.insert(dialog01, {func = AnimCaption, args = {hero.gear, loc("Somewhere on the Planet of Fruits a terrible war is about to begin ..."), 5000}})
 	table.insert(dialog01, {func = AnimSay, args = {hero.gear, loc("I was told that as the leader of the king's guard, no one knows this world better than you!"), SAY_SAY, 5000}})
-	table.insert(dialog01, {func = AnimSay, args = {hero.gear, loc("So, I kindly ask for your help"), SAY_SAY, 3000}})
+	table.insert(dialog01, {func = AnimSay, args = {hero.gear, loc("So, I kindly ask for your help."), SAY_SAY, 3000}})
 	table.insert(dialog01, {func = AnimWait, args = {green1.gear, 2000}})
-	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("You couldn't have come to a worse time Hog Solo!"), SAY_SAY, 3000}})
-	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("The clan of the Red Strawberry wants to take over the dominion and overthrone king Pineapple."), SAY_SAY, 5000}})
-	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("Under normal circumstances we could easily defeat them but we have kindly sent most of our men to the kingdom of Sand to help to the annual dusting of the king's palace."), SAY_SAY, 8000}})
-	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("However the army of Yellow Watermelons is about to attack any moment now."), SAY_SAY, 4000}})
+	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("You couldn't have come to a worse time, Hog Solo!"), SAY_SAY, 3000}})
+	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("The clan of the Red Strawberry wants to take over the dominion and overthrow King Pineapple."), SAY_SAY, 5000}})
+	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("Under normal circumstances we could easily defeat them but we have kindly sent most of our men to the Kingdom of Sand to help with the annual dusting of the king's palace."), SAY_SAY, 8000}})
+	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("However, the army of Yellow Watermelons is about to attack any moment now."), SAY_SAY, 4000}})
 	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("I would gladly help you if we won this battle but under these circumstances I'll only help you if you fight for our side."), SAY_SAY, 6000}})
 	table.insert(dialog01, {func = AnimSay, args = {green1.gear, loc("What do you say? Will you fight for us?"), SAY_SAY, 3000}})
 	table.insert(dialog01, {func = AnimWait, args = {hero.gear, 500}})
-	table.insert(dialog01, {func = ShowMission, args = {missionName, loc("Ready for Battle?"), loc("Walk left if you want to join Captain Lime or right if you want to decline his offer"), 1, 7000}})
-	table.insert(dialog01, {func = AnimSwitchHog, args = {hero.gear}})
+	table.insert(dialog01, {func = ShowMission, args = goals[dialog01]})
+	table.insert(dialog01, {func = AfterDialog01, args = {}})
 	-- DIALOG 02 - Hero selects to fight
 	AddSkipFunction(dialog02, Skipanim, {dialog02})
-	table.insert(dialog02, {func = AnimWait, args = {green1.gear, 3000}})
-	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("You choose well Hog Solo!"), SAY_SAY, 3000}})
-	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("I have only 3 hogs available and they are all cadets"), SAY_SAY, 4000}})
-	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("As you are more experienced, I want you to lead them to the battle"), SAY_SAY, 4000}})
-	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("I of course will observe the battle and intervene if necessary"), SAY_SAY, 5000}})
+	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("You choose well, Hog Solo!"), SAY_SAY, 3000}})
+	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("I have only 3 hogs available and they are all cadets."), SAY_SAY, 4000}})
+	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("As you are more experienced, I want you to lead them to battle."), SAY_SAY, 4000}})
+	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("Of course, I will observe the battle and intervene if necessary."), SAY_SAY, 5000}})
 	table.insert(dialog02, {func = AnimWait, args = {hero.gear, 4500}})
-	table.insert(dialog02, {func = AnimSay, args = {hero.gear, loc("No problem Captain!"), SAY_SAY, 2000}})
+	table.insert(dialog02, {func = AnimSay, args = {hero.gear, loc("No problem, Captain!"), SAY_SAY, 2000}})
 	table.insert(dialog02, {func = AnimSay, args = {hero.gear, loc("The enemies aren't many anyway, it is going to be easy!"), SAY_SAY, 1}})
 	table.insert(dialog02, {func = AnimWait, args = {green1.gear, 9000}})
-	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("Don't be foolish son, there will be more"), SAY_SAY, 2000}})
+	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("Don't be foolish, son, there will be more."), SAY_SAY, 2000}})
 	table.insert(dialog02, {func = AnimSay, args = {green1.gear, loc("Try to be smart and eliminate them quickly. This way you might scare off the rest!"), SAY_SAY, 5000}})
 	table.insert(dialog02, {func = AnimWait, args = {hero.gear, 5000}})
+	table.insert(dialog02, {func = ShowMission, args = goals[dialog02]})
 	table.insert(dialog02, {func = startBattle, args = {hero.gear}})
 	-- DIALOG 03 - Hero selects to flee
 	AddSkipFunction(dialog03, Skipanim, {dialog03})
-	table.insert(dialog03, {func = AnimWait, args = {green1.gear, 3000}})
-	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Too bad... Then you should really leave!"), SAY_SAY, 3000}})
-	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Things are going to get messy around here"), SAY_SAY, 3000}})
-	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Also, you should know that the only place where you can fly is the left-most part of this area"), SAY_SAY, 5000}})
-	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("All the other places are protected by our flight-inhibiting weapons"), SAY_SAY, 4000}})
-	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Now go and don't waste more of my time you coward..."), SAY_SAY, 4000}})
+	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Too bad! Then you should really leave!"), SAY_SAY, 3000}})
+	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Things are going to get messy around here."), SAY_SAY, 3000}})
+	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Also, you should know that the only place where you can fly is the left-most part of this area."), SAY_SAY, 5000}})
+	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("All the other places are protected by our flight-inhibiting weapons."), SAY_SAY, 4000}})
+	table.insert(dialog03, {func = AnimSay, args = {green1.gear, loc("Now go and don't waste more of my time, you coward!"), SAY_SAY, 4000}})
 	table.insert(dialog03, {func = AnimWait, args = {hero.gear, 5000}})
+	table.insert(dialog03, {func = ShowMission, args = goals[dialog03]})
 	table.insert(dialog03, {func = startBattle, args = {hero.gear}})
 end
 
 ------------- OTHER FUNCTIONS ---------------
 
 function startBattle()
+	AnimSetInputMask(0xFFFFFFFF)
 	-- Hog Solo weapons
 	AddAmmo(hero.gear, amRope, 2)
 	AddAmmo(hero.gear, amBazooka, 3)
@@ -437,30 +472,25 @@ function startBattle()
 	AddAmmo(hero.gear, amGrenade, 6)
 	AddAmmo(hero.gear, amDEagle, 4)
 	AddAmmo(hero.gear, amSkip, 100)
-	RestoreHog(green1.bot)
-	DeleteGear(green1.human)
-	green1.gear = green1.bot
+	SetHogLevel(green1.gear, 1)
 	startBattleCalled = true
-	TurnTimeLeft = 0
+	EndTurn(true)
 end
 
 function gameLost()
 	if chooseToBattle then
 		SendStat(siGameResult, loc("The Green Bananas lost, try again!"))
-		SendStat(siCustomAchievement, loc("You have to eliminate all the visible enemies"))
-		SendStat(siCustomAchievement, loc("5 additional enemies will be spawned during the game"))
-		SendStat(siCustomAchievement, loc("You are in control of all the active ally units"))
-		SendStat(siCustomAchievement, loc("The ally units share their ammo"))
-		SendStat(siCustomAchievement, loc("Try to keep as many allies alive as possible"))
+		SendStat(siCustomAchievement, loc("You have to eliminate all the visible enemies."))
+		SendStat(siCustomAchievement, loc("5 additional enemies will be spawned during the game."))
+		SendStat(siCustomAchievement, loc("You are in control of all the active ally units."))
+		SendStat(siCustomAchievement, loc("The ally units share their ammo."))
+		SendStat(siCustomAchievement, loc("Try to keep as many allies alive as possible."))
 	else
 		SendStat(siGameResult, loc("Hog Solo couldn't escape, try again!"))
-		SendStat(siCustomAchievement, loc("You have to get to the left-most land and remove any enemy hog from there"))
-		SendStat(siCustomAchievement, loc("You will play every 3 turns"))
-		SendStat(siCustomAchievement, loc("Green hogs won't intentionally hurt you"))
+		SendStat(siCustomAchievement, loc("You have to get to the left-most land and remove any enemy hog from there."))
+		SendStat(siCustomAchievement, loc("You will play every 3 turns."))
 	end
-	SendStat(siPlayerKills,'1',teamC.name)
-	SendStat(siPlayerKills,'0',teamA.name)
-	SendStat(siPlayerKills,'0',teamB.name)
+	sendSimpleTeamRankings({teamC.name, teamA.name, teamD.name, teamB.name})
 	EndGame()
 end
 
@@ -490,8 +520,28 @@ end
 
 function saveVariables()
 	saveCompletedStatus(2)
-	SaveCampaignVar("UnlockedMissions", "3")
-	SaveCampaignVar("Mission1", "3")
-	SaveCampaignVar("Mission2", "8")
-	SaveCampaignVar("Mission3", "1")
+	SaveCampaignVar("UnlockedMissions", "4")
+	SaveCampaignVar("Mission1", "8")
+	SaveCampaignVar("Mission2", "3")
+	SaveCampaignVar("Mission3", "10")
+	SaveCampaignVar("Mission4", "1")
+end
+
+
+function onLJump()
+	if awaitingInput then
+		PlaySound(sndPlaced)
+		PlaySound(sndCoward, green1.gear)
+		chooseToBattle = false
+		heroSelect()
+	end
+end
+onHJump = onLJump
+
+function onAttack()
+	if awaitingInput then
+		PlaySound(sndPlaced)
+		chooseToBattle = true
+		heroSelect()
+	end
 end
