@@ -1,6 +1,6 @@
 -- Library for localizing strings in lua scripts
 
-local lang = HedgewarsScriptLoad("Locale/" .. tostring(L) .. ".lua")
+local lang = HedgewarsScriptLoad("Locale/" .. tostring(LOCALE) .. ".lua")
 
 function loc(text)
     if locale ~= nil and locale[text] ~= nil then return locale[text]
