@@ -25,13 +25,7 @@ if [ $(which hg) ]; then
 branch=$(hg identify -b)
 rev=$(hg identify -rdefault -i)
 
-#branchurl="<a href=http://code.google.com/p/hedgewars/source/list?name=${branch}>${branch}</a>"
-#revurl="<a href=http://code.google.com/p/hedgewars/source/detail?r=${rev}>${rev}</a>"
-
-branchurl="$branch"
-revurl="$rev"
-
-export PROJECT_NUMBER="${branchurl} branch, ${revurl}"
+export PROJECT_NUMBER="${branch} branch, ${rev}"
 export OUTPUT_DIRECTORY
 
 fi
